/*
/ Objects.cpp
/ miscellaneous ancillary classes implementation
/
/ version 1.2, 2008 October 9
/
/ Author: Sandro Furieri a-furieri@lqt.it
/
/ Copyright (C) 2008  Alessandro Furieri
/
/    This program is free software: you can redistribute it and/or modify
/    it under the terms of the GNU General Public License as published by
/    the Free Software Foundation, either version 3 of the License, or
/    (at your option) any later version.
/
/    This program is distributed in the hope that it will be useful,
/    but WITHOUT ANY WARRANTY; without even the implied warranty of
/    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/    GNU General Public License for more details.
/
/    You should have received a copy of the GNU General Public License
/    along with this program.  If not, see <http://www.gnu.org/licenses/>.
/
*/

#include "Classdef.h"

MyObject::MyObject(int type, wxString & name)
{
//
// constructor - TreeItemData
//
  Type = type;
  Name = name;
  Column = wxT("");
  Temporary = false;
}

MyObject::MyObject(int type, wxString & name, bool tmp)
{
//
// constructor - TreeItemData
//
  Type = type;
  Name = name;
  Column = wxT("");
  Temporary = tmp;
}

MyObject::MyObject(int type, wxString & name, wxString & column)
{
//
// constructor - TreeItemData
//
  Type = type;
  Name = name;
  Column = column;
  Temporary = false;
}

void MyVariant::Copy(MyVariant * other)
{
// 
// transfers a BLOB value
//
  if (other->Type != MY_BLOB_VARIANT)
    return;
  if (!(other->Blob))
    return;
  if (Blob)
    delete[]Blob;
  Type = MY_BLOB_VARIANT;
  BlobSize = other->BlobSize;
  Blob = other->Blob;
  other->Type = MY_NULL_VARIANT;
  other->Blob = NULL;
  other->BlobSize = 0;
}

void MyVariant::Set(const unsigned char *text)
{
//
// sets a String value for this Variant value
//
  Type = MY_TXT_VARIANT;
  TxtValue = wxString::FromUTF8((const char *) text);
}

void MyVariant::Set(const void *blob, int size)
{
//
// sets a BLOB value for this Variant value
//
  if (size <= 0)
    return;
  Type = MY_BLOB_VARIANT;
  BlobSize = size;
  Blob = new unsigned char[size];
  memcpy(Blob, blob, size);
}

void MyRowVariant::Create(int cols)
{
//
// creating the row variant
//
  if (ColumnArray)
    delete[]ColumnArray;
  NumCols = cols;
  ColumnArray = new MyVariant[NumCols];
}

void MyRowVariant::Set(int col, sqlite3_int64 value)
{
//
// setting an Integer value for the Nth column
//
  MyVariant *var;
  if (col < 0 || col >= NumCols)
    return;
  var = ColumnArray + col;
  var->Set(value);
}

void MyRowVariant::Set(int col, double value)
{
//
// setting a Double value for the Nth column
//
  MyVariant *var;
  if (col < 0 || col >= NumCols)
    return;
  var = ColumnArray + col;
  var->Set(value);
}

void MyRowVariant::Set(int col, const unsigned char *value)
{
//
// setting a String value for the Nth column
//
  MyVariant *var;
  if (col < 0 || col >= NumCols)
    return;
  var = ColumnArray + col;
  var->Set(value);
}

void MyRowVariant::Set(int col, const void *blob, int size)
{
//
// setting a BLOB value for the Nth column
//
  MyVariant *var;
  if (col < 0 || col >= NumCols)
    return;
  var = ColumnArray + col;
  var->Set(blob, size);
}

MyVariant *MyRowVariant::GetColumn(int col)
{
//
// returns a Column Variant Value
//
  MyVariant *var;
  if (col < 0 || col >= NumCols)
    return NULL;
  var = ColumnArray + col;
  return var;
}

MyVariantList::MyVariantList()
{
//
// constructor - result set container
//
  NumCols = 0;
  ColumnName = NULL;
  First = NULL;
  Last = NULL;
}

MyVariantList::~MyVariantList()
{
//
// destructor - result set container
//
  Reset();
}

void MyVariantList::Reset(void)
{
// resetting the list to the initial (empty) state
  MyRowVariant *el;
  MyRowVariant *elN;
  if (ColumnName)
    delete[]ColumnName;
  el = First;
  while (el)
    {
      elN = el->GetNext();
      delete el;
      el = elN;
    }
  NumCols = 0;
  ColumnName = NULL;
  First = NULL;
  Last = NULL;
}

int MyVariantList::GetRows()
{
//
// counting how many rows are there
//
  int cnt = 0;
  MyRowVariant *el = First;
  while (el)
    {
      cnt++;
      el = el->GetNext();
    }
  return cnt;
}

MyRowVariant *MyVariantList::Add(int columns)
{
//
// adds a row into the result set
//
  if (!NumCols)
    {
      NumCols = columns;
      ColumnName = new wxString[NumCols];
    }
  MyRowVariant *el = new MyRowVariant(columns);
  if (!First)
    First = el;
  if (Last)
    Last->SetNext(el);
  Last = el;
  return el;
}

void MyVariantList::SetColumnName(int col, const char *name)
{
//
// storing a column name
//
  if (col < 0 || col >= NumCols)
    return;
  ColumnName[col] = wxString::FromUTF8(name);
}

wxString & MyVariantList::GetColumnName(int col)
{
//
// retrieving a column name
//
  return ColumnName[col];
}

MyBlobs::MyBlobs(int rows, int cols)
{
//
// constructor - a BLOB matrix
//
  int r;
  MyRowVariant *rowVar;
  NumRows = 0;
  NumCols = 0;
  Rows = NULL;
  if (rows < 1 || cols < 1)
    return;
  NumRows = rows;
  NumCols = cols;
  Rows = new MyRowVariant[rows];
  for (r = 0; r < rows; r++)
    {
      rowVar = Rows + r;
      rowVar->Create(cols);
    }
}

MyBlobs::~MyBlobs()
{
//
// destructor - a BLOB matrix
//
  if (Rows)
    delete[]Rows;
}

void MyBlobs::SetBlob(int row, int col, MyVariant * org)
{
//
// setting a BLOB value
//
  MyRowVariant *rowVar;
  MyVariant *dest;
  if (row < 0 || row >= NumRows)
    return;
  if (col < 0 || col >= NumCols)
    return;
  rowVar = Rows + row;
  if (!rowVar)
    return;
  dest = rowVar->GetColumn(col);
  if (!dest)
    return;
  dest->Copy(org);
}

MyVariant *MyBlobs::GetBlob(int row, int col)
{
//
// return a BLOB value
//
  MyRowVariant *rowVar;
  MyVariant *dest;
  if (row < 0 || row >= NumRows)
    return NULL;
  if (col < 0 || col >= NumCols)
    return NULL;
  rowVar = Rows + row;
  if (!rowVar)
    return NULL;
  dest = rowVar->GetColumn(col);
  if (!dest)
    return NULL;
  if (dest->GetType() != MY_BLOB_VARIANT)
    return NULL;
  return dest;
}

MyValues::MyValues(int rows, int cols)
{
//
// constructor - a generic values matrix
//
  int r;
  MyRowVariant *rowVar;
  NumRows = 0;
  NumCols = 0;
  Rows = NULL;
  if (rows < 1 || cols < 1)
    return;
  NumRows = rows;
  NumCols = cols;
  Rows = new MyRowVariant[rows];
  for (r = 0; r < rows; r++)
    {
      rowVar = Rows + r;
      rowVar->Create(cols);
    }
}

MyValues::~MyValues()
{
//
// destructor - a generic values matrix
//
  if (Rows)
    delete[]Rows;
}

void MyValues::SetValue(int row, int col, sqlite3_int64 value)
{
//
// setting an integer value
//
  MyRowVariant *rowVar;
  MyVariant *dest;
  if (row < 0 || row >= NumRows)
    return;
  if (col < 0 || col >= NumCols)
    return;
  rowVar = Rows + row;
  if (!rowVar)
    return;
  dest = rowVar->GetColumn(col);
  if (!dest)
    return;
  dest->Set(value);
}

void MyValues::SetValue(int row, int col, double value)
{
//
// setting a double value
//
  MyRowVariant *rowVar;
  MyVariant *dest;
  if (row < 0 || row >= NumRows)
    return;
  if (col < 0 || col >= NumCols)
    return;
  rowVar = Rows + row;
  if (!rowVar)
    return;
  dest = rowVar->GetColumn(col);
  if (!dest)
    return;
  dest->Set(value);
}

void MyValues::SetValue(int row, int col, wxString & value)
{
//
// setting a string value
//
  MyRowVariant *rowVar;
  MyVariant *dest;
  if (row < 0 || row >= NumRows)
    return;
  if (col < 0 || col >= NumCols)
    return;
  rowVar = Rows + row;
  if (!rowVar)
    return;
  dest = rowVar->GetColumn(col);
  if (!dest)
    return;
  dest->Set(value);
}

MyVariant *MyValues::GetValue(int row, int col)
{
//
// return a generic value
//
  MyRowVariant *rowVar;
  MyVariant *dest;
  if (row < 0 || row >= NumRows)
    return NULL;
  if (col < 0 || col >= NumCols)
    return NULL;
  rowVar = Rows + row;
  if (!rowVar)
    return NULL;
  dest = rowVar->GetColumn(col);
  if (!dest)
    return NULL;
  return dest;
}

MyRowVariant *MyValues::GetRow(int row)
{
//
// return a row of generic values
//
  MyRowVariant *rowVar;
  if (row < 0 || row >= NumRows)
    return NULL;
  rowVar = Rows + row;
  return rowVar;
}

MyColumnInfo::MyColumnInfo(wxString & name, bool pkey)
{
//
// constructor - a table column object
//
  Name = name;
  PrimaryKey = pkey;
  Geometry = false;
  GeometryIndex = false;
  MbrCache = false;
  Next = NULL;
}

MyIndexInfo::MyIndexInfo(wxString & name)
{
//
// constructor - a table index object
//
  Name = name;
  Next = NULL;
}

MyTriggerInfo::MyTriggerInfo(wxString & name)
{
//
// constructor - a table trigger object
//
  Name = name;
  Next = NULL;
}

MyTableInfo::~MyTableInfo()
{
//
// destructor - a table columns collection
//
  MyColumnInfo *elc;
  MyColumnInfo *elcN;
  MyIndexInfo *eli;
  MyIndexInfo *eliN;
  MyTriggerInfo *elt;
  MyTriggerInfo *eltN;
  elc = FirstColumn;
  while (elc)
    {
      elcN = elc->GetNext();
      delete elc;
      elc = elcN;
    }
  eli = FirstIndex;
  while (eli)
    {
      eliN = eli->GetNext();
      delete eli;
      eli = eliN;
    }
  elt = FirstTrigger;
  while (elt)
    {
      eltN = elt->GetNext();
      delete elt;
      elt = eltN;
    }
}

void MyTableInfo::AddColumn(wxString & name, bool pkey)
{
//
// inserting a column def into a table
//
  MyColumnInfo *el = new MyColumnInfo(name, pkey);
  if (!FirstColumn)
    FirstColumn = el;
  if (LastColumn)
    LastColumn->SetNext(el);
  LastColumn = el;
}

void MyTableInfo::SetGeometry(wxString & name, bool index, bool cached)
{
//
//setting a geometry column
//
  MyColumnInfo *elc;
  elc = FirstColumn;
  while (elc)
    {
      if (name == elc->GetName())
        {
          elc->SetGeometry();
          if (index == true)
            elc->SetGeometryIndex();
          if (cached == true)
            elc->SetMbrCache();
        }
      elc = elc->GetNext();
    }
}

void MyTableInfo::AddIndex(wxString & name)
{
//
// inserting an index def into a table
//
  MyIndexInfo *el = new MyIndexInfo(name);
  if (!FirstIndex)
    FirstIndex = el;
  if (LastIndex)
    LastIndex->SetNext(el);
  LastIndex = el;
}

void MyTableInfo::AddTrigger(wxString & name)
{
//
// inserting a trigger def into a table
//
  MyTriggerInfo *el = new MyTriggerInfo(name);
  if (!FirstTrigger)
    FirstTrigger = el;
  if (LastTrigger)
    LastTrigger->SetNext(el);
  LastTrigger = el;
}

MyViewInfo::~MyViewInfo()
{
//
// destructor - a view columns collection
//
  MyColumnInfo *elc;
  MyColumnInfo *elcN;
  elc = First;
  while (elc)
    {
      elcN = elc->GetNext();
      delete elc;
      elc = elcN;
    }
}

void MyViewInfo::AddColumn(wxString & name)
{
//
// inserting a column def into a view
//
  MyColumnInfo *el = new MyColumnInfo(name, false);
  if (!First)
    First = el;
  if (Last)
    Last->SetNext(el);
  Last = el;
}

void MyViewInfo::SetGeometry(wxString & name, bool index, bool cached)
{
//
//setting a geometry column
//
  MyColumnInfo *elc;
  elc = First;
  while (elc)
    {
      if (name == elc->GetName())
        {
          elc->SetGeometry();
          if (index == true)
            elc->SetGeometryIndex();
          if (cached == true)
            elc->SetMbrCache();
        }
      elc = elc->GetNext();
    }
}

MySqlHistory::~MySqlHistory()
{
//
// destructor - the SQL queries history
//
  MySqlQuery *elq;
  MySqlQuery *elqN;
  elq = First;
  while (elq)
    {
      elqN = elq->GetNext();
      delete elq;
      elq = elqN;
    }
}

void MySqlHistory::Add(wxString & sql)
{
//
// inserting an SQL query into the history
//
  if (Last)
    {
      // avoiding stupid duplicates
      if (Last->GetSql() == sql)
        return;
    }
  MySqlQuery *el = new MySqlQuery(sql);
  if (!First)
    First = el;
  el->SetPrev(Last);
  if (Last)
    Last->SetNext(el);
  Last = el;
  Current = el;
}

MySqlQuery *MySqlHistory::GetNext()
{
//
// return the next SQL query
//
  if (Current)
    {
      if (Current->GetNext())
        {
          Current = Current->GetNext();
          return Current;
      } else
        return NULL;
    }
  return NULL;
}

MySqlQuery *MySqlHistory::GetPrev()
{
//
// return the previous SQL query
//
  if (Current)
    {
      if (Current->GetPrev())
        {
          Current = Current->GetPrev();
          return Current;
      } else
        return NULL;
    }
  return NULL;
}

bool MySqlHistory::TestNext()
{
//
// tests if the next SQL query exists
//
  if (Current)
    {
      if (Current->GetNext())
        return true;
      else
        return false;
    }
  return false;
}

bool MySqlHistory::TestPrev()
{
//
// tests if the previous SQL query exists
//
  if (Current)
    {
      if (Current->GetPrev())
        return true;
      else
        return false;
    }
  return false;
}

AutoFDOTables::~AutoFDOTables()
{
//
// destructor - auto FDO-OGR wrapper linked list
//
  AutoFDOTable *el;
  AutoFDOTable *elN;
  el = First;
  while (el)
    {
      elN = el->GetNext();
      delete el;
      el = elN;
    }
}

void AutoFDOTables::Add(const char *name, const int len)
{
//
// adding a table name to the auto FDO-OGR wrapper linked list
//
  AutoFDOTable *el = new AutoFDOTable(name, len);
  if (!First)
    First = el;
  if (Last)
    Last->SetNext(el);
  Last = el;
}

GeomColumn::GeomColumn(wxString & name, wxString & type, wxString & dims,
                       int srid, int idx)
{
// Geometry Column - constructor
  GeometryName = name;
  GeometryType = type;
  CoordDims = dims;
  Srid = srid;
  RTree = false;
  MbrCache = false;
  if (idx == 1)
    RTree = true;
  if (idx == 2)
    MbrCache = true;
  NotNull = false;
  Next = NULL;
}

GeomColsList::GeomColsList()
{
// Geometry Columns List - constructor
  First = NULL;
  Last = NULL;
}

GeomColsList::~GeomColsList()
{
// Geometry Columns List - destructor
  GeomColumn *pG;
  GeomColumn *pGn;
  pG = First;
  while (pG)
    {
      pGn = pG->GetNext();
      delete pG;
      pG = pGn;
    }
}

void GeomColsList::Add(wxString & name, wxString & type, wxString & dims,
                       int srid, int idx)
{
// adding a Geometry Column to the list
  GeomColumn *p = new GeomColumn(name, type, dims, srid, idx);
  if (!First)
    First = p;
  if (Last)
    Last->SetNext(p);
  Last = p;
}

void GeomColsList::SetNotNull(wxString & name)
{
// setting a NOT NULL geometry
  GeomColumn *pG = First;
  while (pG)
    {
      if (pG->GetGeometryName() == name)
        {
          pG->SetNotNull();
          return;
        }
      pG = pG->GetNext();
    }
}

IndexColumn::IndexColumn(wxString & name)
{
// Index Column - constructor
  ColumnName = name;
  Valid = true;
  Next = NULL;
}

TblIndex::TblIndex(wxString & name, bool unique)
{
// Table Index - constructor
  IndexName = name;
  Unique = unique;
  Valid = true;
  First = NULL;
  Last = NULL;
  Next = NULL;
}

TblIndex::~TblIndex()
{
// Table Index - destructor
  IndexColumn *pC;
  IndexColumn *pCn;
  pC = First;
  while (pC)
    {
      pCn = pC->GetNext();
      delete pC;
      pC = pCn;
    }
}

void TblIndex::Add(wxString & name)
{
// adding a Column to the Index
  IndexColumn *p = new IndexColumn(name);
  if (!First)
    First = p;
  if (Last)
    Last->SetNext(p);
  Last = p;
}

void TblIndex::Invalidate(wxString & name)
{
// invalidating a column/index
  IndexColumn *pC = First;
  while (pC)
    {
      if (pC->GetColumnName() == name)
        {
          pC->Invalidate();
          Valid = false;
          return;
        }
      pC = pC->GetNext();
    }
}

TblIndexList::TblIndexList()
{
// Table Index List - constructor
  First = NULL;
  Last = NULL;
}

TblIndexList::~TblIndexList()
{
// Table Index List - destructor
  TblIndex *pI;
  TblIndex *pIn;
  pI = First;
  while (pI)
    {
      pIn = pI->GetNext();
      delete pI;
      pI = pIn;
    }
}

void TblIndexList::Add(wxString & name, bool unique)
{
// adding an Index to the List
  TblIndex *p = new TblIndex(name, unique);
  if (!First)
    First = p;
  if (Last)
    Last->SetNext(p);
  Last = p;
}

void TblIndexList::Invalidate(wxString & name)
{
// invalidating a column/index
  TblIndex *pI = First;
  while (pI)
    {
      pI->Invalidate(name);
      pI = pI->GetNext();
    }
}

MyChartData::MyChartData()
{
// constructor (unique values)
  Initialized = false;
  Array = NULL;
  First = NULL;
  Last = NULL;
  MaxFreq = INT_MIN;
  TotFreq = 0;
  MaxClasses = 0;
  NumClasses = 0;
  OtherUniquesFreq = 0;
  OtherUniquesCount = 0;
  Valid = false;
  ByIntervals = false;
}

MyChartData::~MyChartData()
{
// destructor
  MyChartUniqueClass *p;
  MyChartUniqueClass *pN;
  if (Array)
    delete[]Array;
  p = First;
  while (p)
    {
      pN = p->GetNext();
      delete p;
      p = pN;
    }
}

bool MyChartData::Check(bool by_interval, int classes)
{
// checking if current Data are still valid
  if (Initialized == false)
    return false;
  if (by_interval == true && Array == NULL)
    return false;
  if (by_interval == false && First == NULL)
    return false;
  if (by_interval == true && classes != NumClasses)
    return false;
  if (by_interval == false && classes != MaxClasses)
    return false;
  return true;
}

void MyChartData::CleanData()
{
// resetting to the initial (not initialized) state

  MyChartUniqueClass *p;
  MyChartUniqueClass *pN;
  if (Array)
    delete[]Array;
  p = First;
  while (p)
    {
      pN = p->GetNext();
      delete p;
      p = pN;
    }
  Initialized = false;
  Array = NULL;
  First = NULL;
  Last = NULL;
  MaxFreq = INT_MIN;
  TotFreq = 0;
  MaxClasses = 0;
  NumClasses = 0;
  OtherUniquesFreq = 0;
  OtherUniquesCount = 0;
  Valid = false;
  ByIntervals = false;
}

bool MyChartData::Create(int max_classes)
{
// constructor (unique values)
  if (Initialized == true)
    return false;
  MaxClasses = max_classes;
  Valid = false;
  ByIntervals = false;
  return true;
}

bool MyChartData::Create(double min, double max, int classes)
{
// constructor (interval values)
  int ind;
  double base = min;
  double step;
  if (Initialized == true)
    return false;
  Array = new MyChartIntervalClass[classes];
  Min = min;
  Max = max;
  NumClasses = classes;
  Valid = false;
  ByIntervals = true;
  step = (max - min) / (double) classes;
  for (ind = 0; ind < classes; ind++)
    {
      MyChartIntervalClass *p = GetClass(ind);
      p->Create(base, base + step);
      base += step;
    }
  return true;
}

MyChartIntervalClass *MyChartData::GetClass(int idx)
{
// retriving an interval class by index
  if (idx >= 0 && idx < NumClasses)
    return Array + idx;
  return NULL;
}

void MyChartData::Add(double value)
{
// incrementing the frequence count for an interval class
  double step = (Max - Min) / (double) NumClasses;
  int idx = (int) floor((value - Min) / step);
  if (idx >= 0 && idx < NumClasses)
    {
      MyChartIntervalClass *p = GetClass(idx);
      p->Add();
      TotFreq++;
      if (p->GetCount() > MaxFreq)
        MaxFreq = p->GetCount();
    }
}

void MyChartData::Add(wxString & value, int count)
{
// appending a new Unique Value (if limit not already reached)
  if (NumClasses < MaxClasses)
    {
      MyChartUniqueClass *p = new MyChartUniqueClass(value, count);
      if (First == NULL)
        First = p;
      if (Last != NULL)
        Last->SetNext(p);
      Last = p;
      NumClasses++;
      if (count > MaxFreq)
        MaxFreq = count;
  } else
    {
      OtherUniquesFreq += count;
      OtherUniquesCount++;
      if (OtherUniquesFreq > MaxFreq)
        MaxFreq = OtherUniquesFreq;
    }
  TotFreq += count;
}

MyChartScaleLabels::~MyChartScaleLabels()
{
// destructor
  MyChartScaleLabel *p;
  MyChartScaleLabel *pN;
  p = First;
  while (p)
    {
      pN = p->GetNext();
      delete p;
      p = pN;
    }
}

void MyChartScaleLabels::Initialize(double span, int max_freq)
{
// setting the Chart Scale labels
  int tic;
  int tic2;
  char text[128];
  double pos;

  if (max_freq < 20)
    {
      tic = 1;
      while (tic < max_freq)
        {
          sprintf(text, "%d", tic);
          pos = span * ((double) tic / max_freq);
          Add(text, pos);
          tic++;
        }
  } else if (max_freq < 200)
    {
      tic = 10;
      while (tic < max_freq)
        {
          sprintf(text, "%d", tic);
          pos = span * ((double) tic / max_freq);
          Add(text, pos);
          tic += 10;
        }
  } else if (max_freq < 2000)
    {
      tic = 100;
      while (tic < max_freq)
        {
          sprintf(text, "%d", tic);
          pos = span * ((double) tic / max_freq);
          Add(text, pos);
          tic += 100;
        }
  } else if (max_freq < 20000)
    {
      tic = 1000;
      tic2 = 1;
      while (tic < max_freq)
        {
          sprintf(text, "%dK", tic2);
          pos = span * ((double) tic / max_freq);
          Add(text, pos);
          tic += 1000;
          tic2++;
        }
  } else if (max_freq < 200000)
    {
      tic = 10000;
      tic2 = 1;
      while (tic < max_freq)
        {
          sprintf(text, "%d0K", tic2);
          pos = span * ((double) tic / max_freq);
          Add(text, pos);
          tic += 10000;
          tic2++;
        }
  } else if (max_freq < 2000000)
    {
      tic = 100000;
      tic2 = 1;
      while (tic < max_freq)
        {
          sprintf(text, "%d00K", tic2);
          pos = span * ((double) tic / max_freq);
          Add(text, pos);
          tic += 100000;
          tic2++;
        }
  } else if (max_freq < 20000000)
    {
      tic = 1000000;
      tic2 = 1;
      while (tic < max_freq)
        {
          sprintf(text, "%dM", tic2);
          pos = span * ((double) tic / max_freq);
          Add(text, pos);
          tic += 1000000;
          tic2++;
        }
  } else if (max_freq < 200000000)
    {
      tic = 10000000;
      tic2 = 1;
      while (tic < max_freq)
        {
          sprintf(text, "%d0M", tic2);
          pos = span * ((double) tic / max_freq);
          Add(text, pos);
          tic += 10000000;
          tic2++;
        }
  } else if (max_freq < 200000000)
    {
      tic = 100000000;
      tic2 = 1;
      while (tic < max_freq)
        {
          sprintf(text, "%d00M", tic2);
          pos = span * ((double) tic / max_freq);
          Add(text, pos);
          tic += 100000000;
          tic2++;
        }
  } else
    {
      tic = 1000000000;
      tic2 = 1;
      while (tic < max_freq)
        {
          sprintf(text, "%dG", tic2);
          pos = span * ((double) tic / max_freq);
          Add(text, pos);
          tic += 1000000000;
          tic2++;
        }
    }
}

void MyChartScaleLabels::Add(const char *label, double pos)
{
// adding a Scale Label
  wxString text = wxString::FromUTF8(label);
  MyChartScaleLabel *p = new MyChartScaleLabel(text, pos);
  if (First == NULL)
    First = p;
  if (Last != NULL)
    Last->SetNext(p);
  Last = p;
}

MyPieChartLabels::MyPieChartLabels()
{
// constructor
  First = NULL;
  Last = NULL;
  NumLeftLabels = 0;
  LeftLabels = NULL;
  NumRightLabels = 0;
  RightLabels = NULL;
}

MyPieChartLabels::~MyPieChartLabels()
{
// destructor
  MyPieChartLabel *p;
  MyPieChartLabel *pN;
  p = First;
  while (p)
    {
      pN = p->GetNext();
      delete p;
      p = pN;
    }
  if (LeftLabels)
    delete[]LeftLabels;
  if (RightLabels)
    delete[]RightLabels;
}

void MyPieChartLabels::Add(const char *label, double x, double y)
{
// adding a PieChart Label
  wxString text = wxString::FromUTF8(label);
  MyPieChartLabel *p = new MyPieChartLabel(text, x, y);
  if (First == NULL)
    First = p;
  if (Last != NULL)
    Last->SetNext(p);
  Last = p;
}

void MyPieChartLabels::Sort(double cx)
{
// sorting the Left/Right arrays 
  MyPieChartLabel *p;
  MyPieChartLabel *p2;
  int idx;
  bool ok;

  if (LeftLabels)
    delete[]LeftLabels;
  if (RightLabels)
    delete[]RightLabels;
  NumLeftLabels = 0;
  LeftLabels = NULL;
  NumRightLabels = 0;
  RightLabels = NULL;

  p = First;
  while (p)
    {
      // counting how many 'left' labels are there
      if (p->GetX() < cx)
        NumLeftLabels++;
      p = p->GetNext();
    }
  p = First;
  while (p)
    {
      // counting how many 'right' labels are there
      if (p->GetX() >= cx)
        NumRightLabels++;
      p = p->GetNext();
    }
  if (NumLeftLabels > 0)
    {
      // allocating the Left array
      LeftLabels = new MyPieChartLabel *[NumLeftLabels];
      idx = 0;
      p = First;
      while (p)
        {
          // initializing the Left array
          if (p->GetX() < cx)
            {
              *(LeftLabels + idx) = p;
              idx++;
            }
          p = p->GetNext();
        }
      ok = true;
      while (ok)
        {
          // bubble-sorting the Left array
          ok = false;
          for (idx = 1; idx < NumLeftLabels; idx++)
            {
              p = *(LeftLabels + idx - 1);
              p2 = *(LeftLabels + idx);
              if (p->GetY() > p2->GetY())
                {
                  *(LeftLabels + idx - 1) = p2;
                  *(LeftLabels + idx) = p;
                  ok = true;
                }
            }
        }
    }
  if (NumRightLabels > 0)
    {
      // allocating the Right array
      RightLabels = new MyPieChartLabel *[NumRightLabels];
      idx = 0;
      p = First;
      while (p)
        {
          // initializing the Right array
          if (p->GetX() >= cx)
            {
              *(RightLabels + idx) = p;
              idx++;
            }
          p = p->GetNext();
        }
      ok = true;
      while (ok)
        {
          // bubble-sorting the Right array
          ok = false;
          for (idx = 1; idx < NumRightLabels; idx++)
            {
              p = *(RightLabels + idx - 1);
              p2 = *(RightLabels + idx);
              if (p->GetY() > p2->GetY())
                {
                  *(RightLabels + idx - 1) = p2;
                  *(RightLabels + idx) = p;
                  ok = true;
                }
            }
        }
    }
}

MyPieChartLabel *MyPieChartLabels::GetLeftLabel(int idx)
{
// return a Left Label pointer by index
  if (idx >= 0 && idx < NumLeftLabels)
    return *(LeftLabels + idx);
  return NULL;
}

MyPieChartLabel *MyPieChartLabels::GetRightLabel(int idx)
{
// return a Right Label pointer by index
  if (idx >= 0 && idx < NumRightLabels)
    return *(RightLabels + idx);
  return NULL;
}
