/*
/ TableTree.cpp
/ tree control to show tables, columns, indices and triggers
/
/ version 1.2, 2008 October 9
/
/ Author: Sandro Furieri a-furieri@lqt.it
/
/ Copyright (C) 2008  Alessandro Furieri
/
/    This program is free software: you can redistribute it and/or modify
/    it under the terms of the GNU General Public License as published by
/    the Free Software Foundation, either version 3 of the License, or
/    (at your option) any later version.
/
/    This program is distributed in the hope that it will be useful,
/    but WITHOUT ANY WARRANTY; without even the implied warranty of
/    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/    GNU General Public License for more details.
/
/    You should have received a copy of the GNU General Public License
/    along with this program.  If not, see <http://www.gnu.org/licenses/>.
/
*/

#include <float.h>

#include "Classdef.h"

#include "wx/filename.h"
#include "wx/imaglist.h"

#if defined(_WIN32) && !defined(__MINGW32__)
#define strcasecmp	_stricmp
#endif

//
// ICONs in XPM format [universally portable]
//
#include "icons/db.xpm"
#include "icons/table.xpm"
#include "icons/geotable.xpm"
#include "icons/vtable.xpm"
#include "icons/geovtable.xpm"
#include "icons/composer.xpm"
#include "icons/view.xpm"
#include "icons/geoview.xpm"
#include "icons/pkey.xpm"
#include "icons/column.xpm"
#include "icons/index.xpm"
#include "icons/trigger.xpm"
#include "icons/geometry.xpm"
#include "icons/spatialidx.xpm"
#include "icons/mbrcache.xpm"
#include "icons/kill_spindex.xpm"
#include "icons/dumpshp.xpm"
#include "icons/tmp_table.xpm"
#include "icons/tmp_view.xpm"
#include "icons/malformed_geoms.xpm"
#include "icons/statistics.xpm"
#include "icons/map_preview.xpm"

MyTableTree::MyTableTree(MyFrame * parent, wxWindowID id):wxTreeCtrl(parent, id)
{
//
// constructor: TREE control to show DB objects
//
  MainFrame = parent;
  Root = AddRoot(wxT("no current DB"));
// setting up icons 
  Images = new wxImageList(16, 16, true);
  wxIcon icons[17];
  icons[0] = wxIcon(db_xpm);
  icons[1] = wxIcon(table_xpm);
  icons[2] = wxIcon(pkey_xpm);
  icons[3] = wxIcon(column_xpm);
  icons[4] = wxIcon(index_xpm);
  icons[5] = wxIcon(trigger_xpm);
  icons[6] = wxIcon(geometry_xpm);
  icons[7] = wxIcon(spatialidx_xpm);
  icons[8] = wxIcon(vtable_xpm);
  icons[9] = wxIcon(view_xpm);
  icons[10] = wxIcon(geotable_xpm);
  icons[11] = wxIcon(mbrcache_xpm);
  icons[12] = wxIcon(geoview_xpm);
  icons[13] = wxIcon(geovtable_xpm);
  icons[14] = wxIcon(tmp_table_xpm);
  icons[15] = wxIcon(tmp_view_xpm);
  icons[16] = wxIcon(malformed_geoms_xpm);
  Images->Add(icons[0]);
  Images->Add(icons[1]);
  Images->Add(icons[2]);
  Images->Add(icons[3]);
  Images->Add(icons[4]);
  Images->Add(icons[5]);
  Images->Add(icons[6]);
  Images->Add(icons[7]);
  Images->Add(icons[8]);
  Images->Add(icons[9]);
  Images->Add(icons[10]);
  Images->Add(icons[11]);
  Images->Add(icons[12]);
  Images->Add(icons[13]);
  Images->Add(icons[14]);
  Images->Add(icons[15]);
  Images->Add(icons[16]);
  SetImageList(Images);
  SetItemImage(Root, 0);
// setting up event handlers 
  Connect(wxID_ANY, wxEVT_COMMAND_TREE_SEL_CHANGED,
          (wxObjectEventFunction) & MyTableTree::OnSelChanged);
  Connect(wxID_ANY, wxEVT_COMMAND_TREE_ITEM_RIGHT_CLICK,
          (wxObjectEventFunction) & MyTableTree::OnRightClick);
  Connect(Tree_QueryViewComposer, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdQueryViewComposer);
  Connect(Tree_NewTable, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdNewTable);
  Connect(Tree_NewView, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdNewView);
  Connect(Tree_NewIndex, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdNewIndex);
  Connect(Tree_NewTrigger, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdNewTrigger);
  Connect(Tree_NewColumn, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdNewColumn);
  Connect(Tree_Show, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdShow);
  Connect(Tree_Drop, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdDrop);
  Connect(Tree_Rename, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdRename);
  Connect(Tree_Select, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdSelect);
  Connect(Tree_Refresh, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdRefresh);
  Connect(Tree_Recover, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdRecover);
  Connect(Tree_ShowSql, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdShowSql);
  Connect(Tree_SpatialIndex, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdSpatialIndex);
  Connect(Tree_MbrCache, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdMbrCache);
  Connect(Tree_RebuildTriggers, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdRebuildTriggers);
  Connect(Tree_GisLayerAuth, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdGisLayerAuth);
  Connect(Tree_CheckGeometry, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdCheckGeometry);
  Connect(Tree_Extent, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdExtent);
  Connect(Tree_MalformedGeometries, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdMalformedGeometries);
  Connect(Tree_SetSrid, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdSetSrid);
  Connect(Tree_DumpShp, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdDumpShp);
  Connect(Tree_DumpTxtTab, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdDumpTxtTab);
  Connect(Tree_DumpCsv, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdDumpCsv);
  Connect(Tree_DumpHtml, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdDumpHtml);
  Connect(Tree_Edit, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdEdit);
  Connect(Tree_DropColumn, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdDropColumn);
  Connect(Tree_RenameColumn, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdRenameColumn);
  Connect(Tree_ColumnStats, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdColumnStats);
  Connect(Tree_MapPreview, wxEVT_COMMAND_MENU_SELECTED,
          (wxObjectEventFunction) & MyTableTree::OnCmdMapPreview);
}

void MyTableTree::AddTable(wxString & tableName, bool virtualTable, bool tmp)
{
//
// appends a table to the TREE list
  MyTableInfo list;
  MyColumnInfo *col;
  MyIndexInfo *idx;
  MyTriggerInfo *trgr;
  wxTreeItemId item;
  wxTreeItemId item2;
  wxString columnInfo;
  wxString indexInfo;
  wxString triggerInfo;
  int icon = 1;
  if (virtualTable == true)
    icon = 8;
  if (tmp == true)
    icon = 14;
  item = AppendItem(Root, tableName, icon);
  if (virtualTable == true)
    SetItemData(item,
                (wxTreeItemData *) (new MyObject(MY_VTABLE, tableName, tmp)));
  else
    SetItemData(item,
                (wxTreeItemData *) (new MyObject(MY_TABLE, tableName, tmp)));
  MainFrame->GetTableColumns(tableName, &list);
  MainFrame->GetTableIndices(tableName, &list);
  MainFrame->GetTableTriggers(tableName, &list);
  col = list.GetFirstColumn();
  while (col)
    {
      int icon;
      if (col->IsPrimaryKey() == true)
        icon = 2;
      else
        {
          if (col->IsGeometry() == true)
            {
              if (virtualTable == false)
                SetItemImage(item, 10);
              else
                SetItemImage(item, 13);
              if (col->IsGeometryIndex() == true)
                icon = 7;
              else if (col->IsMbrCache() == true)
                icon = 11;
              else
                icon = 6;
          } else
            icon = 3;
        }
      columnInfo = col->GetName();
      item2 = AppendItem(item, columnInfo, icon);
      if (virtualTable == true)
        {
          if (col->IsGeometry() == true)
            {
              SetItemData(item2,
                          (wxTreeItemData *) (new
                                              MyObject(MY_VIRTUAL_GEOMETRY,
                                                       tableName,
                                                       col->GetName())));
          } else
            SetItemData(item2,
                        (wxTreeItemData *) (new
                                            MyObject(MY_VIRTUAL_COLUMN,
                                                     tableName,
                                                     col->GetName())));
      } else
        {
          if (col->IsGeometry() == true)
            {
              if (col->IsGeometryIndex() == true)
                SetItemData(item2,
                            (wxTreeItemData *) (new
                                                MyObject(MY_GEOMETRY_INDEX,
                                                         tableName,
                                                         col->GetName())));
              else if (col->IsMbrCache() == true)
                SetItemData(item2,
                            (wxTreeItemData *) (new
                                                MyObject(MY_GEOMETRY_CACHED,
                                                         tableName,
                                                         col->GetName())));
              else
                SetItemData(item2,
                            (wxTreeItemData *) (new
                                                MyObject(MY_GEOMETRY, tableName,
                                                         col->GetName())));
          } else
            SetItemData(item2,
                        (wxTreeItemData *) (new
                                            MyObject(MY_COLUMN, tableName,
                                                     col->GetName())));
        }
      col = col->GetNext();
    }
  idx = list.GetFirstIndex();
  while (idx)
    {
      indexInfo = idx->GetName();
      item2 = AppendItem(item, indexInfo, 4);
      SetItemData(item2,
                  (wxTreeItemData *) (new MyObject(MY_INDEX, idx->GetName())));
      idx = idx->GetNext();
    }
  trgr = list.GetFirstTrigger();
  while (trgr)
    {
      triggerInfo = trgr->GetName();
      item2 = AppendItem(item, triggerInfo, 5);
      SetItemData(item2,
                  (wxTreeItemData *) (new
                                      MyObject(MY_TRIGGER, trgr->GetName())));
      trgr = trgr->GetNext();
    }
}

void MyTableTree::AddView(wxString & viewName, bool tmp)
{
//
// appends a view to the TREE list
  MyViewInfo list;
  MyColumnInfo *col;
  wxTreeItemId item;
  wxTreeItemId item2;
  wxString columnInfo;
  int icon = 9;
  if (tmp == true)
    icon = 15;
  item = AppendItem(Root, viewName, icon);
  SetItemData(item, (wxTreeItemData *) (new MyObject(MY_VIEW, viewName, tmp)));
  MainFrame->GetViewColumns(viewName, &list);
  col = list.GetFirst();
  while (col)
    {
      columnInfo = col->GetName();
      if (col->IsGeometry() == true)
        {
          SetItemImage(item, 12);
          if (col->IsGeometryIndex() == true)
            icon = 7;
          else if (col->IsMbrCache() == true)
            icon = 11;
          else
            icon = 6;
      } else
        icon = 3;
      item2 = AppendItem(item, columnInfo, icon);
      if (col->IsGeometry() == true)
        {
          if (col->IsGeometryIndex() == true)
            SetItemData(item2,
                        (wxTreeItemData *) (new
                                            MyObject(MY_VIEW_GEOMETRY_INDEX,
                                                     viewName,
                                                     col->GetName())));
          else if (col->IsMbrCache() == true)
            SetItemData(item2,
                        (wxTreeItemData *) (new
                                            MyObject(MY_VIEW_GEOMETRY_CACHED,
                                                     viewName,
                                                     col->GetName())));
          else
            SetItemData(item2,
                        (wxTreeItemData *) (new
                                            MyObject(MY_VIEW_GEOMETRY, viewName,
                                                     col->GetName())));
      } else
        SetItemData(item2,
                    (wxTreeItemData *) (new
                                        MyObject(MY_VIEW_COLUMN, viewName,
                                                 col->GetName())));
      col = col->GetNext();
    }
}

void MyTableTree::OnSelChanged(wxTreeEvent & event)
{
//
// selecting some node [mouse action]
//
  wxTreeItemId item = event.GetItem();
  if (item == Root)
    return;
  MyObject *obj = (MyObject *) GetItemData(item);
  if (obj == NULL)
    return;
  CurrentItem = item;
}

void MyTableTree::OnRightClick(wxTreeEvent & event)
{
//
// right click on some node [mouse action]
//
  wxMenu *menu = new wxMenu();
  wxMenuItem *menuItem;
  wxString title;
  bool table = false;
  bool canEdit = false;
  bool view = false;
  bool column = false;
  bool geometry = false;
  bool geometry_index = false;
  bool geometry_cached = false;
  bool view_column = false;
  bool view_geometry = false;
  bool virtual_column = false;
  bool virtual_geometry = false;
  bool index = false;
  bool trigger = false;
  bool metadata = MainFrame->CheckMetadata();
  if (MainFrame->IsConnected() == false)
    return;
  wxTreeItemId item = event.GetItem();
  SelectItem(item);
  wxPoint pt = event.GetPoint();
  if (item == Root)
    {
      CurrentItem = wxTreeItemId();
      menuItem = new wxMenuItem(menu, Tree_Refresh, wxT("&Refresh"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_QueryViewComposer,
                       wxT("Query/View &Composer"));
      menuItem->SetBitmap(wxBitmap(composer_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_NewTable, wxT("Create New &Table"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_NewView, wxT("Create New &View"));
      menu->Append(menuItem);
      PopupMenu(menu, pt);
      return;
    }
  MyObject *obj = (MyObject *) GetItemData(item);
  if (obj == NULL)
    {
      CurrentItem = wxTreeItemId();
      menuItem = new wxMenuItem(menu, Tree_Refresh, wxT("&Refresh"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_QueryViewComposer,
                       wxT("Query/View &Composer"));
      menuItem->SetBitmap(wxBitmap(composer_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_NewTable, wxT("Create New &Table"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_NewView, wxT("Create New &View"));
      menu->Append(menuItem);
      PopupMenu(menu, pt);
      return;
    }
  switch (obj->GetType())
    {
      case MY_VTABLE:
      case MY_TABLE:
        table = true;
        break;
      case MY_VIEW:
        view = true;
        break;
      case MY_COLUMN:
        column = true;
        break;
      case MY_GEOMETRY:
        geometry = true;
        break;
      case MY_GEOMETRY_INDEX:
        geometry_index = true;
        break;
      case MY_GEOMETRY_CACHED:
        geometry_cached = true;
        break;
      case MY_VIEW_COLUMN:
        view_column = true;
        break;
      case MY_VIEW_GEOMETRY:
      case MY_VIEW_GEOMETRY_INDEX:
      case MY_VIEW_GEOMETRY_CACHED:
        view_geometry = true;
        break;
      case MY_VIRTUAL_COLUMN:
        virtual_column = true;
        break;
      case MY_VIRTUAL_GEOMETRY:
        virtual_geometry = true;
        break;
      case MY_INDEX:
        index = true;
        break;
      case MY_TRIGGER:
        trigger = true;
        break;
    };
  if (obj->GetType() == MY_TABLE)
    canEdit = true;
  CurrentItem = item;
  menuItem = new wxMenuItem(menu, Tree_Refresh, wxT("&Refresh"));
  menu->Append(menuItem);
  if (table == true)
    {
      wxString title = wxT("Table: ") + obj->GetName();
      menu->SetTitle(title);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_QueryViewComposer,
                       wxT("Query/View &Composer"));
      menuItem->SetBitmap(wxBitmap(composer_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_NewTable, wxT("Create New &Table"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_NewView, wxT("Create New &View"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      if (canEdit == true)
        {
          menuItem = new wxMenuItem(menu, Tree_Edit, wxT("&Edit table rows"));
          menu->Append(menuItem);
      } else
        {
          menuItem = new wxMenuItem(menu, Tree_Select, wxT("&Query table"));
          menu->Append(menuItem);
        }
      menuItem = new wxMenuItem(menu, Tree_Show, wxT("&Show columns"));
      menu->Append(menuItem);
      menuItem =
        new wxMenuItem(menu, Tree_ShowSql, wxT("&Show CREATE statement"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_NewColumn, wxT("Add New &Column"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_Rename, wxT("&Rename table"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_Drop, wxT("&Drop table"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_NewIndex, wxT("Create New &Index"));
      menu->Append(menuItem);
      menuItem =
        new wxMenuItem(menu, Tree_NewTrigger, wxT("Create New &Trigger"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_DumpTxtTab, wxT("Export as &Txt/Tab"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_DumpCsv, wxT("Export as &CSV"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_DumpHtml, wxT("Export as &HTML"));
      menu->Append(menuItem);
    }
  if (view == true)
    {
      wxString title = wxT("View: ") + obj->GetName();
      menu->SetTitle(title);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_QueryViewComposer,
                       wxT("Query/View &Composer"));
      menuItem->SetBitmap(wxBitmap(composer_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_NewTable, wxT("Create New &Table"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_NewView, wxT("Create New &View"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_Select, wxT("&Query view"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_Show, wxT("&Show columns"));
      menu->Append(menuItem);
      menuItem =
        new wxMenuItem(menu, Tree_ShowSql, wxT("&Show CREATE statement"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_Drop, wxT("&Drop view"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_DumpTxtTab, wxT("Export as &Txt/Tab"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_DumpCsv, wxT("Export as &CSV"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_DumpHtml, wxT("Export as &HTML"));
      menu->Append(menuItem);
    }
  if (column == true)
    {
      wxString title =
        wxT("Column: ") + obj->GetName() + wxT(".") + obj->GetColumn();
      menu->SetTitle(title);
      menu->AppendSeparator();
      if (MainFrame->IsPrimaryKey(obj->GetName(), obj->GetColumn()) == false)
        {
          menuItem = new wxMenuItem(menu, Tree_DropColumn, wxT("&Drop Column"));
          menu->Append(menuItem);
          menuItem =
            new wxMenuItem(menu, Tree_RenameColumn, wxT("&Rename Column"));
          menu->Append(menuItem);
          menu->AppendSeparator();
        }
      menuItem =
        new wxMenuItem(menu, Tree_CheckGeometry, wxT("&Check geometries"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_MapPreview, wxT("&Map Preview"));
      menuItem->SetBitmap(wxBitmap(map_preview_xpm));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_Extent, wxT("&Extent"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_SetSrid, wxT("&Set SRID"));
      menu->Append(menuItem);
      if (metadata == true)
        {
          menu->AppendSeparator();
          menuItem =
            new wxMenuItem(menu, Tree_Recover, wxT("&Recover geometry column"));
          menu->Append(menuItem);
        }
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_ColumnStats, wxT("&Statistic snapshot"));
      menuItem->SetBitmap(wxBitmap(statistics_xpm));
      menu->Append(menuItem);
    }
  if (view_column == true)
    {
      wxString title =
        wxT("Column: ") + obj->GetName() + wxT(".") + obj->GetColumn();
      menu->SetTitle(title);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_ColumnStats, wxT("&Statistic snapshot"));
      menuItem->SetBitmap(wxBitmap(statistics_xpm));
      menu->Append(menuItem);
    }
  if (virtual_column == true)
    {
      wxString title =
        wxT("Column: ") + obj->GetName() + wxT(".") + obj->GetColumn();
      menu->SetTitle(title);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_ColumnStats, wxT("&Statistic snapshot"));
      menuItem->SetBitmap(wxBitmap(statistics_xpm));
      menu->Append(menuItem);
    }
  if (geometry == true)
    {
      wxString title =
        wxT("Column: ") + obj->GetName() + wxT(".") + obj->GetColumn();
      menu->SetTitle(title);
      menuItem = new wxMenuItem(menu, Tree_Show, wxT("&Show Spatial Metadata"));
      menu->Append(menuItem);
      menuItem =
        new wxMenuItem(menu, Tree_GisLayerAuth,
                       wxT("&GIS layer authorizations"));
      menu->Append(menuItem);
      menuItem =
        new wxMenuItem(menu, Tree_CheckGeometry, wxT("&Check geometries"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_Extent, wxT("&Extent"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_SpatialIndex, wxT("&Build Spatial Index"));
      menuItem->SetBitmap(wxBitmap(spatialidx_xpm));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_MbrCache, wxT("Build &MBR cache"));
      menuItem->SetBitmap(wxBitmap(mbrcache_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_RebuildTriggers,
                       wxT("Rebuild Geometry &Triggers"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_DumpShp, wxT("Export as &Shapefile"));
      menuItem->SetBitmap(wxBitmap(dumpshp_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_MalformedGeometries,
                       wxT("&Malformed geometries"));
      menuItem->SetBitmap(wxBitmap(malformed_geoms_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_MapPreview, wxT("&Map Preview"));
      menuItem->SetBitmap(wxBitmap(map_preview_xpm));
      menu->Append(menuItem);
    }
  if (geometry_index == true)
    {
      wxString title =
        wxT("Column: ") + obj->GetName() + wxT(".") + obj->GetColumn();
      menu->SetTitle(title);
      menuItem = new wxMenuItem(menu, Tree_Show, wxT("&Show Spatial Metadata"));
      menu->Append(menuItem);
      menuItem =
        new wxMenuItem(menu, Tree_GisLayerAuth,
                       wxT("&GIS layer authorizations"));
      menu->Append(menuItem);
      menuItem =
        new wxMenuItem(menu, Tree_CheckGeometry, wxT("&Check geometries"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_Extent, wxT("&Extent"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_SpatialIndex, wxT("&Remove Spatial Index"));
      menuItem->SetBitmap(wxBitmap(kill_spindex_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_RebuildTriggers,
                       wxT("Rebuild Geometry &Triggers"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_DumpShp, wxT("Export as &Shapefile"));
      menuItem->SetBitmap(wxBitmap(dumpshp_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_MalformedGeometries,
                       wxT("&Malformed geometries"));
      menuItem->SetBitmap(wxBitmap(malformed_geoms_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_MapPreview, wxT("&Map Preview"));
      menuItem->SetBitmap(wxBitmap(map_preview_xpm));
      menu->Append(menuItem);
    }
  if (geometry_cached == true)
    {
      wxString title =
        wxT("Column: ") + obj->GetName() + wxT(".") + obj->GetColumn();
      menu->SetTitle(title);
      menuItem = new wxMenuItem(menu, Tree_Show, wxT("&Show Spatial Metadata"));
      menu->Append(menuItem);
      menuItem =
        new wxMenuItem(menu, Tree_GisLayerAuth,
                       wxT("&GIS layer authorizations"));
      menu->Append(menuItem);
      menuItem =
        new wxMenuItem(menu, Tree_CheckGeometry, wxT("&Check geometries"));
      menu->Append(menuItem);
      menuItem = new wxMenuItem(menu, Tree_Extent, wxT("&Extent"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_MbrCache, wxT("&Remove MBR cache"));
      menuItem->SetBitmap(wxBitmap(kill_spindex_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_RebuildTriggers,
                       wxT("Rebuild Geometry &Triggers"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_DumpShp, wxT("Export as &Shapefile"));
      menuItem->SetBitmap(wxBitmap(dumpshp_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_MalformedGeometries,
                       wxT("&Malformed geometries"));
      menuItem->SetBitmap(wxBitmap(malformed_geoms_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_MapPreview, wxT("&Map Preview"));
      menuItem->SetBitmap(wxBitmap(map_preview_xpm));
      menu->Append(menuItem);
    }
  if (view_geometry == true)
    {
      wxString title =
        wxT("Column: ") + obj->GetName() + wxT(".") + obj->GetColumn();
      menu->SetTitle(title);
      menuItem = new wxMenuItem(menu, Tree_Show, wxT("&Show Spatial Metadata"));
      menu->Append(menuItem);
      menuItem =
        new wxMenuItem(menu, Tree_GisLayerAuth,
                       wxT("&GIS layer authorizations"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_DumpShp, wxT("Export as &Shapefile"));
      menuItem->SetBitmap(wxBitmap(dumpshp_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_MalformedGeometries,
                       wxT("&Malformed geometries"));
      menuItem->SetBitmap(wxBitmap(malformed_geoms_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_MapPreview, wxT("&Map Preview"));
      menuItem->SetBitmap(wxBitmap(map_preview_xpm));
      menu->Append(menuItem);
    }
  if (virtual_geometry == true)
    {
      wxString title =
        wxT("Column: ") + obj->GetName() + wxT(".") + obj->GetColumn();
      menu->SetTitle(title);
      menuItem = new wxMenuItem(menu, Tree_Show, wxT("&Show Spatial Metadata"));
      menu->Append(menuItem);
      menuItem =
        new wxMenuItem(menu, Tree_GisLayerAuth,
                       wxT("&GIS layer authorizations"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem =
        new wxMenuItem(menu, Tree_MalformedGeometries,
                       wxT("&Malformed geometries"));
      menuItem->SetBitmap(wxBitmap(malformed_geoms_xpm));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_MapPreview, wxT("&Map Preview"));
      menuItem->SetBitmap(wxBitmap(map_preview_xpm));
      menu->Append(menuItem);
    }
  if (index == true)
    {
      wxString title = wxT("Index: ") + obj->GetName();
      menu->SetTitle(title);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_Show, wxT("&Show index"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_Drop, wxT("&Drop index"));
      menu->Append(menuItem);
    }
  if (trigger == true)
    {
      wxString title = wxT("Trigger: ") + obj->GetName();
      menu->SetTitle(title);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_Show, wxT("&Show trigger"));
      menu->Append(menuItem);
      menu->AppendSeparator();
      menuItem = new wxMenuItem(menu, Tree_Drop, wxT("&Drop trigger"));
      menu->Append(menuItem);
    }
  PopupMenu(menu, pt);
}

void MyTableTree::OnCmdQueryViewComposer(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - query/view composer invoked
//
  MainFrame->QueryViewComposer();
}

void MyTableTree::OnCmdNewTable(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - new table creation required
//
  wxString sql;
  sql = wxT("CREATE TABLE ...table-name... (\n");
  sql += wxT("...column1,\n...column2,\n...columnN)");
  MainFrame->SetSql(sql, false);
}

void MyTableTree::OnCmdNewView(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - new view creation required
//
  wxString sql;
  sql = wxT("CREATE VIEW ...view-name... AS\n");
  sql += wxT("SELECT ...sql-select-statement...");
  MainFrame->SetSql(sql, false);
}

void MyTableTree::OnCmdNewIndex(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - new index creation required
//
  wxString sql;
  char xname[1024];
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_TABLE)
    {
      sql = wxT("CREATE [ UNIQUE ] INDEX ...index-name...\nON ");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("\n(\n...column1, column2, columnN...\n)");
      MainFrame->SetSql(sql, false);
    }
}

void MyTableTree::OnCmdNewTrigger(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - new trigger creation required
//
  wxString sql;
  char xname[1024];
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_TABLE)
    {
      sql = wxT("CREATE TRIGGER ...trigger-name...\n[ BEFORE | AFTER ]\n");
      sql += wxT("[ INSERT | UPDATE | DELETE ]\nON ");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("\n...sql-statement...");
      MainFrame->SetSql(sql, false);
    }
}

void MyTableTree::OnCmdNewColumn(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - new column creation required
//
  wxString sql;
  char xname[1024];
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_TABLE)
    {
      sql = wxT("ALTER TABLE ");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("\nADD COLUMN ...column-name column-type...");
      MainFrame->SetSql(sql, false);
    }
}

void MyTableTree::OnCmdSelect(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - examining table rows required
//
  wxString sql;
  char xname[1024];
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  sql = wxT("SELECT * FROM ");
  strcpy(xname, obj->GetName().ToUTF8());
  MainFrame->DoubleQuotedSql(xname);
  sql += wxString::FromUTF8(xname);
  MainFrame->SetSql(sql, true);
}

void MyTableTree::OnCmdShow(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - examining full infos required
//
  wxString sql;
  char xname[1024];
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_TABLE || obj->GetType() == MY_VTABLE
      || obj->GetType() == MY_VIEW)
    {
      sql = wxT("PRAGMA table_info(");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT(")");
    }
  if (obj->GetType() == MY_GEOMETRY || obj->GetType() == MY_GEOMETRY_INDEX
      || obj->GetType() == MY_GEOMETRY_CACHED)
    {
      sql = wxT("SELECT *\nFROM geom_cols_ref_sys\n");
      sql += wxT("WHERE f_table_name = '");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("'\nAND f_geometry_column = '");
      strcpy(xname, obj->GetColumn().ToUTF8());
      MainFrame->CleanSqlString(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("'");
    }
  if (obj->GetType() == MY_VIEW_GEOMETRY
      || obj->GetType() == MY_VIEW_GEOMETRY_INDEX
      || obj->GetType() == MY_VIEW_GEOMETRY_CACHED)
    {
      sql = wxT("SELECT *\nFROM views_geometry_columns\n");
      sql +=
        wxT("JOIN geometry_columns USING (f_table_name, f_geometry_column)\n");
      sql += wxT("JOIN spatial_ref_sys USING (srid)\n");
      sql += wxT("WHERE view_name = '");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("'\nAND view_geometry = '");
      strcpy(xname, obj->GetColumn().ToUTF8());
      MainFrame->CleanSqlString(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("'");
    }
  if (obj->GetType() == MY_VIRTUAL_GEOMETRY)
    {
      sql = wxT("SELECT *\nFROM virts_geometry_columns\n");
      sql += wxT("JOIN spatial_ref_sys USING (srid)\n");
      sql += wxT("WHERE virt_name = '");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("'\nAND virt_geometry = '");
      strcpy(xname, obj->GetColumn().ToUTF8());
      MainFrame->CleanSqlString(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("'");
    }
  if (obj->GetType() == MY_INDEX)
    {
      sql = wxT("PRAGMA index_info(");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT(")");
    }
  if (obj->GetType() == MY_TRIGGER)
    {
      sql =
        wxT
        ("SELECT sql FROM sqlite_master\nWHERE type = 'trigger' AND name = '");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("'");
    }
  if (sql.Len() < 1)
    return;
  MainFrame->SetSql(sql, true);
}

void MyTableTree::OnCmdDrop(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - dropping some object required
//
  wxString sql;
  wxString sql2;
  wxString msg;
  char *errMsg = NULL;
  int ret;
  int i;
  char **results;
  int rows;
  int columns;
  bool metadata = false;
  wxString name;
  char xname[1024];
  sqlite3 *sqlite = MainFrame->GetSqlite();
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_TABLE || obj->GetType() == MY_VTABLE)
    {
      metadata = MainFrame->CheckMetadata();
      if (metadata == true)
        {
          // SpatialMetadata exists; we must check for geometry_columns and SpatialIndices as well
          sqlite = MainFrame->GetSqlite();
          sql = wxT("SELECT f_geometry_column, spatial_index_enabled ");
          sql += wxT("FROM geometry_columns WHERE f_table_name = '");
          strcpy(xname, obj->GetName().ToUTF8());
          MainFrame->CleanSqlString(xname);
          sql += wxString::FromUTF8(xname);
          sql += wxT("'");
          ret =
            sqlite3_get_table(sqlite, sql.ToUTF8(), &results, &rows, &columns,
                              &errMsg);
          if (ret != SQLITE_OK)
            {
              wxMessageBox(wxT("SQLite SQL error: ") +
                           wxString::FromUTF8(errMsg), wxT("spatialite-gui"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_free(errMsg);
              return;
            }
          sql = wxT("BEGIN; ");
          if (rows < 1)
            ;
          else
            {
              for (i = 1; i <= rows; i++)
                {
                  if (results[(i * columns) + 0])
                    {
                      const char *column = results[(i * columns) + 0];
                      wxString geo_column = wxString::FromUTF8(column);
                      if (atoi(results[(i * columns) + 1]))
                        {
                          // dropping the SpatialIndex
                          sql += wxT("DROP TABLE IF EXISTS ");
                          name = wxT("idx_");
                          name += obj->GetName();
                          name += wxT("_");
                          name += geo_column;
                          strcpy(xname, name.ToUTF8());
                          MainFrame->DoubleQuotedSql(xname);
                          sql += wxString::FromUTF8(xname);
                          sql += wxT("; ");
                          // dropping the MBR cache
                          sql += wxT("DROP TABLE IF EXISTS ");
                          name = wxT("cache_");
                          name += obj->GetName();
                          name += wxT("_");
                          name += geo_column;
                          strcpy(xname, name.ToUTF8());
                          MainFrame->DoubleQuotedSql(xname);
                          sql += wxString::FromUTF8(xname);
                          sql += wxT("; ");
                        }
                      // deleting from geometry_columns
                      sql +=
                        wxT
                        ("DELETE FROM geometry_columns\n    WHERE f_table_name = '");
                      strcpy(xname, obj->GetName().ToUTF8());
                      MainFrame->CleanSqlString(xname);
                      sql += wxString::FromUTF8(xname);
                      sql += wxT("'\n        AND f_geometry_column = '");
                      strcpy(xname, geo_column.ToUTF8());
                      MainFrame->CleanSqlString(xname);
                      sql += wxString::FromUTF8(xname);
                      sql += wxT("'; ");
                    }
                }
            }
          sqlite3_free_table(results);
          sql += wxT("DROP TABLE IF EXISTS ");
          strcpy(xname, obj->GetName().ToUTF8());
          MainFrame->DoubleQuotedSql(xname);
          sql += wxString::FromUTF8(xname);
          sql += wxT("; COMMIT;");
      } else
        {
          // there are no SpatialMetadata at all; we'll simply try to drop the table
          sql = wxT("BEGIN; ");
          sql += wxT("DROP TABLE IF EXISTS ");
          strcpy(xname, obj->GetName().ToUTF8());
          MainFrame->DoubleQuotedSql(xname);
          sql += wxString::FromUTF8(xname);
          sql += wxT("; COMMIT;");
        }
      msg = wxT("Do you really intend to drop the Table named: ");
      msg += obj->GetName();
      msg += wxT("\n[and any other related object] ?");
    }
  if (obj->GetType() == MY_VIEW)
    {
      msg = wxT("Do you really intend to drop the View named: ");
      msg += obj->GetName();
      msg += wxT(" ?");
      sql = wxT("BEGIN; DROP VIEW IF EXISTS ");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("; COMMIT;");
    }
  if (obj->GetType() == MY_INDEX)
    {
      msg = wxT("Do you really intend to drop the Index named: ");
      msg += obj->GetName();
      msg += wxT(" ?");
      sql = wxT("BEGIN; DROP INDEX IF EXISTS ");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("; COMMIT;");
    }
  if (obj->GetType() == MY_TRIGGER)
    {
      msg = wxT("Do you really intend to drop the Trigger named: ");
      msg += obj->GetName();
      msg += wxT(" ?");
      sql = wxT("BEGIN; DROP TRIGGER IF EXISTS ");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("; COMMIT;");
    }
  if (sql.Len() < 1)
    return;
  wxMessageDialog confirm(this, msg, wxT("Confirming DROP"),
                          wxOK | wxCANCEL | wxICON_QUESTION);
  ret = confirm.ShowModal();
  if (ret != wxID_OK)
    return;
  ::wxBeginBusyCursor();
  if (obj->GetType() == MY_TABLE || obj->GetType() == MY_VTABLE)
    {
      if (metadata == true)
        {
          // deleting from GIS tables
          sql2 = wxT("SELECT f_geometry_column, spatial_index_enabled ");
          sql2 += wxT("FROM geometry_columns WHERE f_table_name = '");
          strcpy(xname, obj->GetName().ToUTF8());
          MainFrame->CleanSqlString(xname);
          sql2 += wxString::FromUTF8(xname);
          sql2 += wxT("'");
          ret =
            sqlite3_get_table(sqlite, sql2.ToUTF8(), &results, &rows, &columns,
                              &errMsg);
          if (ret != SQLITE_OK)
            {
              wxMessageBox(wxT("SQLite SQL error: ") +
                           wxString::FromUTF8(errMsg), wxT("spatialite-gui"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_free(errMsg);
          } else
            {
              if (rows < 1)
                ;
              else
                {
                  for (i = 1; i <= rows; i++)
                    {
                      if (results[(i * columns) + 0])
                        {
                          const char *column = results[(i * columns) + 0];
                          wxString geo_column = wxString::FromUTF8(column);
                          MainFrame->DeleteFromGisTables(obj->GetName(),
                                                         geo_column);
                        }
                    }
                }
            }
          sqlite3_free_table(results);
        }
    }
  ret = sqlite3_exec(sqlite, sql.ToUTF8(), NULL, NULL, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      ::wxEndBusyCursor();
      goto rollback;
    }
  ::wxEndBusyCursor();
  wxMessageBox(wxT("Selected object '") + obj->GetName() +
               wxT("' was successfully removed"), wxT("spatialite-gui"),
               wxOK | wxICON_INFORMATION, this);
  MainFrame->InitTableTree();
  return;
rollback:
  ret = sqlite3_exec(sqlite, "ROLLBACK", NULL, NULL, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      ::wxEndBusyCursor();
      return;
    }
  ::wxEndBusyCursor();
  wxMessageBox(wxT
               ("An error occurred\n\na ROLLBACK was automatically performed"),
               wxT("spatialite-gui"), wxOK | wxICON_WARNING, this);
}

void MyTableTree::OnCmdRename(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - table renaming required
//
  wxString sql;
  char xname[1024];
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_TABLE || obj->GetType() == MY_VTABLE)
    {
      sql = wxT("ALTER TABLE ");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("\nRENAME TO ...new-table-name...");
      MainFrame->SetSql(sql, false);
    }
}

void MyTableTree::OnCmdRefresh(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - refreshing the Tree
//
  MainFrame->InitTableTree();
}

void MyTableTree::OnCmdRecover(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - Recover Geometry
//
  char *errMsg = NULL;
  int ret;
  wxString sql;
  int i;
  char **results;
  int rows;
  int columns;
  int retval = 0;
  int srid = -1;
  char dummy[128];
  wxString type;
  wxString dims;
  RecoverDialog dlg;
  char xname[1024];
  sqlite3 *sqlite = MainFrame->GetSqlite();
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_COLUMN)
    {
      // trying to recover a geometry column
      if (MainFrame->CheckMetadata() == false)
        {
          wxMessageBox(wxT
                       ("Missing Spatial Metadata tables\n\ntry to run the 'init_spatialite.sql' script ..."),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          return;
        }
      dlg.Create(MainFrame, obj->GetName(), obj->GetColumn());
      ret = dlg.ShowModal();
      if (ret == wxID_OK)
        {
          srid = dlg.GetSrid();
          type = dlg.GetType();
          dims = dlg.GetDimension();
          ::wxBeginBusyCursor();
          ret = sqlite3_exec(sqlite, "BEGIN", NULL, NULL, &errMsg);
          if (ret != SQLITE_OK)
            {
              wxMessageBox(wxT("SQLite SQL error: ") +
                           wxString::FromUTF8(errMsg), wxT("spatialite-gui"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_free(errMsg);
              ::wxEndBusyCursor();
              return;
            }
          sql = wxT("SELECT RecoverGeometryColumn('");
          strcpy(xname, obj->GetName().ToUTF8());
          MainFrame->CleanSqlString(xname);
          sql += wxString::FromUTF8(xname);
          sql += wxT("', '");
          strcpy(xname, obj->GetColumn().ToUTF8());
          MainFrame->CleanSqlString(xname);
          sql += wxString::FromUTF8(xname);
          sprintf(dummy, "', %d, '", srid);
          sql += wxString::FromUTF8(dummy);
          sql += type;
          sql += wxT("', '");
          sql += dims;
          sql += wxT("')");
          ret =
            sqlite3_get_table(sqlite, sql.ToUTF8(), &results, &rows, &columns,
                              &errMsg);
          if (ret != SQLITE_OK)
            {
              wxMessageBox(wxT("SQLite SQL error: ") +
                           wxString::FromUTF8(errMsg), wxT("spatialite-gui"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_free(errMsg);
              goto rollback;
            }
          if (rows < 1)
            ;
          else
            {
              for (i = 1; i <= rows; i++)
                {
                  if (results[(i * columns) + 0])
                    retval = atoi(results[(i * columns) + 0]);
                }
            }
          sqlite3_free_table(results);
          if (!retval)
            goto rollback;
          ret = sqlite3_exec(sqlite, "COMMIT", NULL, NULL, &errMsg);
          if (ret != SQLITE_OK)
            {
              wxMessageBox(wxT("SQLite SQL error: ") +
                           wxString::FromUTF8(errMsg), wxT("spatialite-gui"),
                           wxOK | wxICON_ERROR, this);
              sqlite3_free(errMsg);
              ::wxEndBusyCursor();
              return;
            }
          ::wxEndBusyCursor();
          wxMessageBox(wxT("Geometry column ") + obj->GetName() +
                       wxT(".") + obj->GetColumn() +
                       wxT(" was successfully recovered"),
                       wxT("spatialite-gui"), wxOK | wxICON_INFORMATION, this);
        }
    }
  return;
rollback:
  ret = sqlite3_exec(sqlite, "ROLLBACK", NULL, NULL, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      ::wxEndBusyCursor();
      return;
    }
  ::wxEndBusyCursor();
  wxMessageBox(wxT
               ("Geometry column doesn't satisfies required constraints\n\na ROLLBACK was automatically performed"),
               wxT("spatialite-gui"), wxOK | wxICON_WARNING, this);
}

void MyTableTree::OnCmdShowSql(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - Showing CREATE TABLE statement
//
  wxString sql;
  char xname[1024];
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_TABLE || obj->GetType() == MY_VTABLE)
    {
      sql = wxT("SELECT sql FROM ");
      if (obj->IsTemporary() == true)
        sql += wxT("sqlite_temp_master");
      else
        sql += wxT("sqlite_master");
      sql += wxT("\nWHERE type = 'table' AND name = '");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("'");
      MainFrame->SetSql(sql, true);
    }
  if (obj->GetType() == MY_VIEW)
    {
      sql = wxT("SELECT sql FROM ");
      if (obj->IsTemporary() == true)
        sql += wxT("sqlite_temp_master");
      else
        sql += wxT("sqlite_master");
      sql += wxT("\nWHERE type = 'view' AND name = '");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("'");
      MainFrame->SetSql(sql, true);
    }
}

void MyTableTree::OnCmdSpatialIndex(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - Spatial Index creation-destruction
//
  char *errMsg = NULL;
  int ret;
  wxString sql;
  wxString msg;
  int i;
  char **results;
  int rows;
  int columns;
  int retval = 0;
  wxString name;
  char xname[1024];
  char xtable[1024];
  char xcolumn[1024];
  sqlite3 *sqlite = MainFrame->GetSqlite();
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_GEOMETRY)
    {
      // creating the Spatial Index
      ::wxBeginBusyCursor();
      ret = sqlite3_exec(sqlite, "BEGIN", NULL, NULL, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          ::wxEndBusyCursor();
          return;
        }
      strcpy(xtable, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xtable);
      strcpy(xcolumn, obj->GetColumn().ToUTF8());
      MainFrame->CleanSqlString(xcolumn);
      sql = wxT("SELECT CreateSpatialIndex('");
      sql += wxString::FromUTF8(xtable);
      sql += wxT("', '");
      sql += wxString::FromUTF8(xcolumn);
      sql += wxT("')");
      ret =
        sqlite3_get_table(sqlite, sql.ToUTF8(), &results, &rows, &columns,
                          &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          goto rollback;
        }
      if (rows < 1)
        ;
      else
        {
          for (i = 1; i <= rows; i++)
            {
              if (results[(i * columns) + 0])
                retval = atoi(results[(i * columns) + 0]);
            }
        }
      sqlite3_free_table(results);
      if (!retval)
        goto rollback;
      ret = sqlite3_exec(sqlite, "COMMIT", NULL, NULL, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          ::wxEndBusyCursor();
          return;
        }
      ::wxEndBusyCursor();
      wxMessageBox(wxT("Spatial Index idx_") + obj->GetName() +
                   wxT("_") + obj->GetColumn() +
                   wxT(" was successfully created"), wxT("spatialite-gui"),
                   wxOK | wxICON_INFORMATION, this);
      MainFrame->InitTableTree();
  } else if (obj->GetType() == MY_GEOMETRY_INDEX)
    {
      // dropping the Spatial Index
      ::wxBeginBusyCursor();
      ret = sqlite3_exec(sqlite, "BEGIN", NULL, NULL, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          ::wxEndBusyCursor();
          return;
        }
      strcpy(xtable, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xtable);
      strcpy(xcolumn, obj->GetColumn().ToUTF8());
      MainFrame->CleanSqlString(xcolumn);
      sql = wxT("SELECT DisableSpatialIndex('");
      sql += wxString::FromUTF8(xtable);
      sql += wxT("', '");
      sql += wxString::FromUTF8(xcolumn);
      sql += wxT("')");
      msg = wxT("Do you really intend to delete the SpatialIndex\n");
      msg += wxT("on column ");
      msg += obj->GetName();
      msg += wxT(".");
      msg += obj->GetColumn();
      msg += wxT(" ?");
      wxMessageDialog confirm(this, msg, wxT("Confirming Delete Spatial Index"),
                              wxOK | wxCANCEL | wxICON_QUESTION);
      ret = confirm.ShowModal();
      if (ret != wxID_OK)
        return;
      ret =
        sqlite3_get_table(sqlite, sql.ToUTF8(), &results, &rows, &columns,
                          &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          goto rollback;
        }
      if (rows < 1)
        ;
      else
        {
          for (i = 1; i <= rows; i++)
            {
              if (results[(i * columns) + 0])
                retval = atoi(results[(i * columns) + 0]);
            }
        }
      sqlite3_free_table(results);
      if (!retval)
        goto rollback;
      sql = wxT("DROP TABLE IF EXISTS ");
      name = wxT("idx_");
      name += obj->GetName();
      name += wxT("_");
      name += obj->GetColumn();
      strcpy(xname, name.ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      ret = sqlite3_exec(sqlite, sql.ToUTF8(), NULL, NULL, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          ::wxEndBusyCursor();
          return;
        }
      ret = sqlite3_exec(sqlite, "COMMIT", NULL, NULL, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          ::wxEndBusyCursor();
          return;
        }
      ::wxEndBusyCursor();
      wxMessageBox(wxT("Spatial Index idx_") + obj->GetName() +
                   wxT("_") + obj->GetColumn() +
                   wxT(" was successfully removed"), wxT("spatialite-gui"),
                   wxOK | wxICON_INFORMATION, this);
      MainFrame->InitTableTree();
    }
  return;
rollback:
  ret = sqlite3_exec(sqlite, "ROLLBACK", NULL, NULL, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      ::wxEndBusyCursor();
      return;
    }
  ::wxEndBusyCursor();
  wxMessageBox(wxT
               ("An error occurred\n\na ROLLBACK was automatically performed"),
               wxT("spatialite-gui"), wxOK | wxICON_WARNING, this);
}

void MyTableTree::OnCmdMbrCache(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - MBR cache creation-destruction
//
  char *errMsg = NULL;
  int ret;
  wxString sql;
  wxString msg;
  int i;
  char **results;
  int rows;
  int columns;
  int retval = 0;
  wxString name;
  char xname[1024];
  char xtable[1024];
  char xcolumn[1024];
  sqlite3 *sqlite = MainFrame->GetSqlite();
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_GEOMETRY)
    {
      // creating the MBR cache
      ::wxBeginBusyCursor();
      ret = sqlite3_exec(sqlite, "BEGIN", NULL, NULL, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          ::wxEndBusyCursor();
          return;
        }
      strcpy(xtable, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xtable);
      strcpy(xcolumn, obj->GetColumn().ToUTF8());
      MainFrame->CleanSqlString(xcolumn);
      sql = wxT("SELECT CreateMbrCache('");
      sql += wxString::FromUTF8(xtable);
      sql += wxT("', '");
      sql += wxString::FromUTF8(xcolumn);
      sql += wxT("')");
      ret =
        sqlite3_get_table(sqlite, sql.ToUTF8(), &results, &rows, &columns,
                          &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          goto rollback;
        }
      if (rows < 1)
        ;
      else
        {
          for (i = 1; i <= rows; i++)
            {
              if (results[(i * columns) + 0])
                retval = atoi(results[(i * columns) + 0]);
            }
        }
      sqlite3_free_table(results);
      if (!retval)
        goto rollback;
      ret = sqlite3_exec(sqlite, "COMMIT", NULL, NULL, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          ::wxEndBusyCursor();
          return;
        }
      ::wxEndBusyCursor();
      wxMessageBox(wxT("MBR cache cache_") + obj->GetName() +
                   wxT("_") + obj->GetColumn() +
                   wxT(" was successfully created"), wxT("spatialite-gui"),
                   wxOK | wxICON_INFORMATION, this);
      MainFrame->InitTableTree();
  } else if (obj->GetType() == MY_GEOMETRY_CACHED)
    {
      // dropping the MBR cache
      ::wxBeginBusyCursor();
      ret = sqlite3_exec(sqlite, "BEGIN", NULL, NULL, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          ::wxEndBusyCursor();
          return;
        }
      strcpy(xtable, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xtable);
      strcpy(xcolumn, obj->GetColumn().ToUTF8());
      MainFrame->CleanSqlString(xcolumn);
      sql = wxT("SELECT DisableSpatialIndex('");
      sql += wxString::FromUTF8(xtable);
      sql += wxT("', '");
      sql += wxString::FromUTF8(xcolumn);
      sql += wxT("')");
      msg = wxT("Do you really intend to delete the MBR cache\n");
      msg += wxT("on column ");
      msg += obj->GetName();
      msg += wxT(".");
      msg += obj->GetColumn();
      msg += wxT(" ?");
      wxMessageDialog confirm(this, msg, wxT("Confirming Delete MBR cache"),
                              wxOK | wxCANCEL | wxICON_QUESTION);
      ret = confirm.ShowModal();
      if (ret != wxID_OK)
        return;
      ret =
        sqlite3_get_table(sqlite, sql.ToUTF8(), &results, &rows, &columns,
                          &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          goto rollback;
        }
      if (rows < 1)
        ;
      else
        {
          for (i = 1; i <= rows; i++)
            {
              if (results[(i * columns) + 0])
                retval = atoi(results[(i * columns) + 0]);
            }
        }
      sqlite3_free_table(results);
      if (!retval)
        goto rollback;
      sql = wxT("DROP TABLE IF EXISTS ");
      name = wxT("cache_");
      name += obj->GetName();
      name += wxT("_");
      name += obj->GetColumn();
      strcpy(xname, name.ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      ret = sqlite3_exec(sqlite, sql.ToUTF8(), NULL, NULL, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          ::wxEndBusyCursor();
          return;
        }
      ret = sqlite3_exec(sqlite, "COMMIT", NULL, NULL, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          ::wxEndBusyCursor();
          return;
        }
      ::wxEndBusyCursor();
      wxMessageBox(wxT("MBR cache cache_") + obj->GetName() +
                   wxT("_") + obj->GetColumn() +
                   wxT(" was successfully removed"), wxT("spatialite-gui"),
                   wxOK | wxICON_INFORMATION, this);
      MainFrame->InitTableTree();
    }
  return;
rollback:
  ret = sqlite3_exec(sqlite, "ROLLBACK", NULL, NULL, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      ::wxEndBusyCursor();
      return;
    }
  ::wxEndBusyCursor();
  wxMessageBox(wxT
               ("An error occurred\n\na ROLLBACK was automatically performed"),
               wxT("spatialite-gui"), wxOK | wxICON_WARNING, this);
}

void MyTableTree::OnCmdRebuildTriggers(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - rebuilding Geometry Triggers
//
  wxString sql;
  char xtable[1024];
  char xcolumn[1024];
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_COLUMN || obj->GetType() == MY_GEOMETRY
      || obj->GetType() == MY_GEOMETRY_INDEX
      || obj->GetType() == MY_GEOMETRY_CACHED)
    {
      strcpy(xtable, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xtable);
      strcpy(xcolumn, obj->GetColumn().ToUTF8());
      MainFrame->CleanSqlString(xcolumn);
      sql = wxT("SELECT RebuildGeometryTriggers('");
      sql += wxString::FromUTF8(xtable);
      sql += wxT("', '");
      sql += wxString::FromUTF8(xcolumn);
      sql += wxT("')");
      MainFrame->SetSql(sql, true);
    }
}

void MyTableTree::OnCmdGisLayerAuth(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - setting the GIS Layer auth
//
  wxString sql;
  int ret;
  char **results;
  int rows;
  int columns;
  int i;
  char *errMsg = NULL;
  char *value;
  bool readOnly = false;
  bool hidden = false;
  GisLayerAuthDialog dlg;
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_GEOMETRY || obj->GetType() == MY_GEOMETRY_INDEX
      || obj->GetType() == MY_GEOMETRY_CACHED
      || obj->GetType() == MY_VIEW_GEOMETRY)
    {
      sql = wxT("SELECT read_only, hidden FROM geometry_columns ");
      sql += wxT("LEFT JOIN geometry_columns_auth USING ");
      sql += wxT("(f_table_name, f_geometry_column) WHERE f_table_name = '");
      sql += obj->GetName();
      sql += wxT("' AND f_geometry_column = '");
      sql += obj->GetColumn();
      sql += wxT("'");
    }
  if (obj->GetType() == MY_VIEW_GEOMETRY_INDEX
      || obj->GetType() == MY_VIEW_GEOMETRY_CACHED
      || obj->GetType() == MY_VIRTUAL_GEOMETRY)
    {
      sql = wxT("SELECT read_only, hidden FROM views_geometry_columns ");
      sql += wxT("LEFT JOIN geometry_columns_auth AS x ON ");
      sql +=
        wxT
        ("(view_name = x.f_table_name AND view_geometry = x.f_geometry_column) ");
      sql += wxT("WHERE view_name = '");
      sql += obj->GetName();
      sql += wxT("' AND view_geometry = '");
      sql += obj->GetColumn();
      sql += wxT("'");
    }
  if (obj->GetType() == MY_VIRTUAL_GEOMETRY)
    {
      sql = wxT("SELECT read_only, hidden FROM virts_geometry_columns ");
      sql += wxT("LEFT JOIN geometry_columns_auth ON ");
      sql +=
        wxT
        ("(virt_name = f_table_name AND virt_geometry = f_geometry_column) ");
      sql += wxT("WHERE virt_name = '");
      sql += obj->GetName();
      sql += wxT("' AND virt_geometry = '");
      sql += obj->GetColumn();
      sql += wxT("'");
    }
  if (sql.Len() == 0)
    return;
  ret = sqlite3_get_table(MainFrame->GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      return;
    }
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          value = results[(i * columns) + 0];
          if (value)
            {
              if (atoi(value) == 0)
                readOnly = false;
              else
                readOnly = true;
            }
          value = results[(i * columns) + 1];
          if (value)
            {
              if (atoi(value) == 0)
                hidden = false;
              else
                hidden = true;
            }
        }
    }
  sqlite3_free_table(results);
  dlg.Create(MainFrame, obj->GetName(), obj->GetColumn(), readOnly, hidden);
  ret = dlg.ShowModal();
  if (ret == wxID_OK)
    {
      // updating the GEOMETRY_COLUMNS_AUTH table
      sql = wxT("INSERT OR REPLACE INTO geometry_columns_auth ");
      sql += wxT("(f_table_name, f_geometry_column, read_only, hidden) ");
      sql += wxT("VALUES ('");
      sql += obj->GetName();
      sql += wxT("', '");
      sql += obj->GetColumn();
      if (dlg.IsHidden() == true)
        sql += wxT("', 0, 1)");
      else if (dlg.IsReadOnly() == true)
        sql += wxT("', 1, 0)");
      else
        sql += wxT("', 0, 0)");
      ret =
        sqlite3_exec(MainFrame->GetSqlite(), sql.ToUTF8(), NULL, NULL, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
        }
    }
}

void MyTableTree::OnCmdCheckGeometry(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - checking geometries
//
  wxString sql;
  char xname[1024];
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_COLUMN || obj->GetType() == MY_GEOMETRY
      || obj->GetType() == MY_GEOMETRY_INDEX
      || obj->GetType() == MY_GEOMETRY_CACHED)
    {
      sql = wxT("SELECT Count(*), GeometryType(");
      strcpy(xname, obj->GetColumn().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("), Srid(");
      strcpy(xname, obj->GetColumn().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("), CoordDimension(");
      strcpy(xname, obj->GetColumn().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT(")\nFROM ");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("\nGROUP BY 2, 3, 4");
      MainFrame->SetSql(sql, true);
    }
}

void MyTableTree::OnCmdExtent(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - computing Extent
//
  wxString sql;
  char xname[1024];
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_COLUMN || obj->GetType() == MY_GEOMETRY
      || obj->GetType() == MY_GEOMETRY_INDEX
      || obj->GetType() == MY_GEOMETRY_CACHED)
    {
      sql = wxT("SELECT Min(MbrMinX(");
      strcpy(xname, obj->GetColumn().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT(")), Min(MbrMinY(");
      strcpy(xname, obj->GetColumn().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT(")), Max(MbrMaxX(");
      strcpy(xname, obj->GetColumn().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT(")), Max(MbrMaxY(");
      strcpy(xname, obj->GetColumn().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("))\nFROM ");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      MainFrame->SetSql(sql, true);
    }
}

void MyTableTree::OnCmdMalformedGeometries(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - identifying malformed geometries
//
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_GEOMETRY || obj->GetType() == MY_GEOMETRY_INDEX
      || obj->GetType() == MY_GEOMETRY_CACHED
      || obj->GetType() == MY_VIEW_GEOMETRY
      || obj->GetType() == MY_VIEW_GEOMETRY_INDEX
      || obj->GetType() == MY_VIEW_GEOMETRY_CACHED
      || obj->GetType() == MY_VIRTUAL_GEOMETRY)
    {
      bool repair = true;
      if (obj->GetType() == MY_VIEW_GEOMETRY
          || obj->GetType() == MY_VIEW_GEOMETRY_INDEX
          || obj->GetType() == MY_VIEW_GEOMETRY_CACHED
          || obj->GetType() == MY_VIRTUAL_GEOMETRY)
        repair = false;
      MalformedGeomsList *list =
        MainFrame->FindMalformedGeoms(obj->GetName(), obj->GetColumn(), repair);
      if (list->GetFirst() == NULL)
        {
          delete list;
          wxString msg = wxT("Any geometry found in ");
          msg += obj->GetName();
          msg += wxT(".");
          msg += obj->GetColumn();
          msg += wxT(" is valid");
          wxMessageBox(msg, wxT("spatialite-gui"), wxOK | wxICON_INFORMATION,
                       this);
          return;
      } else
        {
          MalformedGeomsDialog dlg;
          dlg.Create(MainFrame, obj->GetName(), obj->GetColumn(), list);
          dlg.ShowModal();
        }
    }
}

void MyTableTree::OnCmdSetSrid(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - setting SRID for geometries
//
  SetSridDialog dlg;
  wxString sql;
  int srid;
  int oldSrid;
  int ret;
  char dummy[128];
  char xname[1024];
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_COLUMN)
    {
      dlg.Create(MainFrame, obj->GetName(), obj->GetColumn());
      ret = dlg.ShowModal();
      if (ret == wxID_OK)
        {
          srid = dlg.GetSrid();
          oldSrid = dlg.GetOldSrid();
          sql = wxT("UPDATE ");
          strcpy(xname, obj->GetName().ToUTF8());
          MainFrame->DoubleQuotedSql(xname);
          sql += wxString::FromUTF8(xname);
          sql += wxT(" SET ");
          strcpy(xname, obj->GetColumn().ToUTF8());
          MainFrame->DoubleQuotedSql(xname);
          sql += wxString::FromUTF8(xname);
          sql += wxT(" = SetSrid(");
          strcpy(xname, obj->GetColumn().ToUTF8());
          MainFrame->DoubleQuotedSql(xname);
          sql += wxString::FromUTF8(xname);
          sprintf(dummy, ", %d)", srid);
          sql += wxString::FromUTF8(dummy);
          sql += wxT("\nWHERE Srid(");
          strcpy(xname, obj->GetColumn().ToUTF8());
          MainFrame->DoubleQuotedSql(xname);
          sql += wxString::FromUTF8(xname);
          sprintf(dummy, ") = %d", oldSrid);
          sql += wxString::FromUTF8(dummy);
          MainFrame->SetSql(sql, true);
        }
    }
}

void MyTableTree::OnCmdDumpShp(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - dumping as Shapefile
//
  int ret;
  wxString path;
  wxString lastDir;
  bool isView = false;
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  if (obj->GetType() == MY_VIEW_GEOMETRY
      || obj->GetType() == MY_VIEW_GEOMETRY_INDEX
      || obj->GetType() == MY_VIEW_GEOMETRY_CACHED)
    isView = true;
  if (obj->GetType() == MY_GEOMETRY || obj->GetType() == MY_GEOMETRY_INDEX
      || obj->GetType() == MY_GEOMETRY_CACHED || isView == true)
    {
      wxFileDialog fileDialog(this, wxT("Dump Shapefile"),
                              wxT(""), wxT("shapefile.shp"),
                              wxT
                              ("Shapefile (*.shp)|*.shp|All files (*.*)|*.*"),
                              wxFD_SAVE | wxFD_OVERWRITE_PROMPT,
                              wxDefaultPosition, wxDefaultSize, wxT("filedlg"));
      lastDir = MainFrame->GetLastDirectory();
      if (lastDir.Len() >= 1)
        fileDialog.SetDirectory(lastDir);
      ret = fileDialog.ShowModal();
      if (ret == wxID_OK)
        {

          wxFileName file(fileDialog.GetPath());
          path = file.GetPath();
          path += file.GetPathSeparator();
          path += file.GetName();
          lastDir = file.GetPath();
          if (MainFrame->IsSetAskCharset() == false)
            {
              // using the default output charset
              MainFrame->SetLastDirectory(lastDir);
              ::wxBeginBusyCursor();
              MainFrame->DumpShapefile(path, obj->GetName(), obj->GetColumn(),
                                       MainFrame->GetDefaultCharset(), isView,
                                       false);
              ::wxEndBusyCursor();
          } else
            {
              // asking the charset to be used
              DumpShpDialog dlg;
              dlg.Create(MainFrame, path, obj->GetName(), obj->GetColumn(),
                         MainFrame->GetDefaultCharset());
              ret = dlg.ShowModal();
              if (ret == wxID_OK)
                {
                  MainFrame->SetLastDirectory(lastDir);
                  ::wxBeginBusyCursor();
                  MainFrame->DumpShapefile(path, obj->GetName(),
                                           obj->GetColumn(), dlg.GetCharset(),
                                           isView, false);
                  ::wxEndBusyCursor();
                }
            }
        }
    }
}

void MyTableTree::OnCmdDumpTxtTab(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - dumping as TxtTab
//
  int ret;
  wxString path;
  wxString lastDir;
  wxString target;
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  wxFileDialog fileDialog(this, wxT("Dump Txt/Tab file"),
                          wxT(""), wxT("table.txt"),
                          wxT("File Txt/Tab (*.txt)|*.txt|All files (*.*)|*.*"),
                          wxFD_SAVE | wxFD_OVERWRITE_PROMPT,
                          wxDefaultPosition, wxDefaultSize, wxT("filedlg"));
  lastDir = MainFrame->GetLastDirectory();
  if (lastDir.Len() >= 1)
    fileDialog.SetDirectory(lastDir);
  ret = fileDialog.ShowModal();
  if (ret == wxID_OK)
    {
      wxFileName file(fileDialog.GetPath());
      path = file.GetPath();
      path += file.GetPathSeparator();
      path += file.GetName();
      path += wxT(".txt");
      lastDir = file.GetPath();
      if (MainFrame->IsSetAskCharset() == false)
        {
          // using the default output charset
          MainFrame->SetLastDirectory(lastDir);
          ::wxBeginBusyCursor();
          MainFrame->DumpTxtTab(path, obj->GetName(),
                                MainFrame->GetDefaultCharset());
          ::wxEndBusyCursor();
      } else
        {
          // asking the charset to be used
          DumpTxtDialog dlg;
          target = wxT("TXT / TAB");
          dlg.Create(MainFrame, path, target, MainFrame->GetDefaultCharset());
          ret = dlg.ShowModal();
          if (ret == wxID_OK)
            {
              MainFrame->SetLastDirectory(lastDir);
              ::wxBeginBusyCursor();
              MainFrame->DumpTxtTab(path, obj->GetName(), dlg.GetCharset());
              ::wxEndBusyCursor();
            }
        }
    }
}

void MyTableTree::OnCmdDumpCsv(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - dumping as CSV
//
  int ret;
  wxString path;
  wxString lastDir;
  wxString target;
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  wxFileDialog fileDialog(this, wxT("Dump CSV file"),
                          wxT(""), wxT("table.csv"),
                          wxT("File CSV (*.csv)|*.csv|All files (*.*)|*.*"),
                          wxFD_SAVE | wxFD_OVERWRITE_PROMPT,
                          wxDefaultPosition, wxDefaultSize, wxT("filedlg"));
  lastDir = MainFrame->GetLastDirectory();
  if (lastDir.Len() >= 1)
    fileDialog.SetDirectory(lastDir);
  ret = fileDialog.ShowModal();
  if (ret == wxID_OK)
    {
      wxFileName file(fileDialog.GetPath());
      path = file.GetPath();
      path += file.GetPathSeparator();
      path += file.GetName();
      path += wxT(".csv");
      lastDir = file.GetPath();
      if (MainFrame->IsSetAskCharset() == false)
        {
          // using the default output charset
          MainFrame->SetLastDirectory(lastDir);
          ::wxBeginBusyCursor();
          MainFrame->DumpCsv(path, obj->GetName(),
                             MainFrame->GetDefaultCharset());
          ::wxEndBusyCursor();
      } else
        {
          // asking the charset to be used
          DumpTxtDialog dlg;
          target = wxT("CSV");
          dlg.Create(MainFrame, path, target, MainFrame->GetDefaultCharset());
          ret = dlg.ShowModal();
          if (ret == wxID_OK)
            {
              MainFrame->SetLastDirectory(lastDir);
              ::wxBeginBusyCursor();
              MainFrame->DumpCsv(path, obj->GetName(), dlg.GetCharset());
              ::wxEndBusyCursor();
            }
        }
    }
}

void MyTableTree::OnCmdDumpHtml(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - dumping as Html
//
  int ret;
  wxString path;
  wxString lastDir;
  wxString target;
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  wxFileDialog fileDialog(this, wxT("Dump HTML file"),
                          wxT(""), wxT("table.html"),
                          wxT
                          ("HTML web page (*.html)|*.html|All files (*.*)|*.*"),
                          wxFD_SAVE | wxFD_OVERWRITE_PROMPT, wxDefaultPosition,
                          wxDefaultSize, wxT("filedlg"));
  lastDir = MainFrame->GetLastDirectory();
  if (lastDir.Len() >= 1)
    fileDialog.SetDirectory(lastDir);
  ret = fileDialog.ShowModal();
  if (ret == wxID_OK)
    {
      wxFileName file(fileDialog.GetPath());
      path = file.GetPath();
      path += file.GetPathSeparator();
      path += file.GetName();
      path += wxT(".html");
      lastDir = file.GetPath();
      if (MainFrame->IsSetAskCharset() == false)
        {
          // using the default output charset
          MainFrame->SetLastDirectory(lastDir);
          ::wxBeginBusyCursor();
          MainFrame->DumpHtml(path, obj->GetName(),
                              MainFrame->GetDefaultCharset());
          ::wxEndBusyCursor();
      } else
        {
          // asking the charset to be used
          DumpTxtDialog dlg;
          target = wxT("HTML");
          dlg.Create(MainFrame, path, target, MainFrame->GetDefaultCharset());
          ret = dlg.ShowModal();
          if (ret == wxID_OK)
            {
              MainFrame->SetLastDirectory(lastDir);
              ::wxBeginBusyCursor();
              MainFrame->DumpHtml(path, obj->GetName(), dlg.GetCharset());
              ::wxEndBusyCursor();
            }
        }
    }
}

void MyTableTree::OnCmdMapPreview(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - Map Preview
//
  char table_name[1024];
  char column_name[1024];
  char sql[4192];
  char sql2[2048];
  int ret;
  char err_msg[2048];
  sqlite3_stmt *stmt;
  double minx = DBL_MAX;
  double miny = DBL_MAX;
  double maxx = DBL_MAX;
  double maxy = DBL_MAX;

  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;

  ::wxBeginBusyCursor();
  strcpy(table_name, obj->GetName().ToUTF8());
  strcpy(column_name, obj->GetColumn().ToUTF8());
  MainFrame->DoubleQuotedSql(table_name);
  MainFrame->DoubleQuotedSql(column_name);
  sprintf(sql, "SELECT Min(MbrMinX(%s)), Min(MbrMinY(%s)), ", column_name,
          column_name);
  sprintf(sql2, "Max(MbrMaxX(%s)), Max(MbrMaxY(%s)) ", column_name,
          column_name);
  strcat(sql, sql2);
  sprintf(sql2, "FROM %s", table_name);
  strcat(sql, sql2);

  ret =
    sqlite3_prepare_v2(MainFrame->GetSqlite(), sql, strlen(sql), &stmt, NULL);
  if (ret != SQLITE_OK)
    {
      sprintf(err_msg, "SQL error: %s", sqlite3_errmsg(MainFrame->GetSqlite()));
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(err_msg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      ::wxEndBusyCursor();
      return;
    }
  while (1)
    {
      //
      // fetching the result set rows 
      //
      ret = sqlite3_step(stmt);
      if (ret == SQLITE_DONE)
        break;                  // end of result set
      if (ret == SQLITE_ROW)
        {
          //
          // fetching a row
          //
          if (sqlite3_column_type(stmt, 0) == SQLITE_FLOAT)
            minx = sqlite3_column_double(stmt, 0);
          if (sqlite3_column_type(stmt, 1) == SQLITE_FLOAT)
            miny = sqlite3_column_double(stmt, 1);
          if (sqlite3_column_type(stmt, 2) == SQLITE_FLOAT)
            maxx = sqlite3_column_double(stmt, 2);
          if (sqlite3_column_type(stmt, 3) == SQLITE_FLOAT)
            maxy = sqlite3_column_double(stmt, 3);
      } else
        {
          sqlite3_finalize(stmt);
          sprintf(err_msg, "SQL error: %s",
                  sqlite3_errmsg(MainFrame->GetSqlite()));
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(err_msg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          ::wxEndBusyCursor();
          return;
        }
    }
  sqlite3_finalize(stmt);
  ::wxEndBusyCursor();

  if (minx == DBL_MAX || miny == DBL_MAX || maxx == DBL_MAX || maxy == DBL_MAX)
    {
      wxMessageBox(wxT("This Column doesn't contains any Geometry: sorry ..."),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      return;
    }

  MapPreviewDialog dlg;
  dlg.Create(MainFrame, obj->GetName(), obj->GetColumn(), minx, miny, maxx,
             maxy);
  dlg.ShowModal();
}

void MyTableTree::OnCmdColumnStats(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - column stats
//
  char table_name[1024];
  char column_name[1024];
  char sql[4192];
  char sql2[4192];
  int ret;
  char **results;
  int rows;
  int columns;
  int i;
  char *errMsg = NULL;
  const char *value;
  char err_msg[2048];
  sqlite3_stmt *stmt;
  int count;
  int null_count = 0;
  int text_count = 0;
  int integer_count = 0;
  int real_count = 0;
  int blob_count = 0;
  double min;
  double max;
  double avg;
  double stddev_pop;
  double stddev_samp;
  double var_pop;
  double var_samp;
  int distinct_values = 0;

  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;

  ::wxBeginBusyCursor();
  strcpy(table_name, obj->GetName().ToUTF8());
  strcpy(column_name, obj->GetColumn().ToUTF8());
  MainFrame->DoubleQuotedSql(table_name);
  MainFrame->DoubleQuotedSql(column_name);
  sprintf(sql, "SELECT Typeof(%s), Count(*) FROM %s GROUP BY Typeof(%s)",
          column_name, table_name, column_name);

  ret = sqlite3_get_table(MainFrame->GetSqlite(), sql, &results,
                          &rows, &columns, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      ::wxEndBusyCursor();
      return;
    }
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          value = results[(i * columns) + 0];
          count = atoi(results[(i * columns) + 1]);
          if (strcasecmp(value, "null") == 0)
            null_count += count;
          if (strcasecmp(value, "text") == 0)
            text_count += count;
          if (strcasecmp(value, "integer") == 0)
            integer_count += count;
          if (strcasecmp(value, "real") == 0)
            real_count += count;
          if (strcasecmp(value, "blob") == 0)
            blob_count += count;
        }
    }
  sqlite3_free_table(results);

  if ((real_count + integer_count) > 0)
    {
      // computing statistic analysis
      sprintf(sql, "SELECT Min(%s), Max(%s), Avg(%s), ", column_name,
              column_name, column_name);
      sprintf(sql2, "StdDev_pop(%s), StdDev_samp(%s), ", column_name,
              column_name);
      strcat(sql, sql2);
      sprintf(sql2, "Var_pop(%s), Var_samp(%s) FROM %s", column_name,
              column_name, table_name);
      strcat(sql, sql2);

      ret = sqlite3_get_table(MainFrame->GetSqlite(), sql, &results,
                              &rows, &columns, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          ::wxEndBusyCursor();
          return;
        }
      if (rows < 1)
        ;
      else
        {
          for (i = 1; i <= rows; i++)
            {
              min = atof(results[(i * columns) + 0]);
              max = atof(results[(i * columns) + 1]);
              avg = atof(results[(i * columns) + 2]);
              stddev_pop = atof(results[(i * columns) + 3]);
              stddev_samp = atof(results[(i * columns) + 4]);
              var_pop = atof(results[(i * columns) + 5]);
              var_samp = atof(results[(i * columns) + 6]);
            }
        }
      sqlite3_free_table(results);
    }
// computing DISTINCT values
  sprintf(sql, "SELECT DISTINCT %s FROM %s", column_name, table_name);
  ret =
    sqlite3_prepare_v2(MainFrame->GetSqlite(), sql, strlen(sql), &stmt, NULL);
  if (ret != SQLITE_OK)
    {
      sprintf(err_msg, "SQL error: %s", sqlite3_errmsg(MainFrame->GetSqlite()));
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(err_msg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      ::wxEndBusyCursor();
      return;
    }
  while (1)
    {
      //
      // fetching the result set rows 
      //
      ret = sqlite3_step(stmt);
      if (ret == SQLITE_DONE)
        break;                  // end of result set
      if (ret == SQLITE_ROW)
        {
          //
          // fetching a row
          //
          distinct_values++;
      } else
        {
          sqlite3_finalize(stmt);
          sprintf(err_msg, "SQL error: %s",
                  sqlite3_errmsg(MainFrame->GetSqlite()));
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(err_msg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          ::wxEndBusyCursor();
          return;
        }
    }
  sqlite3_finalize(stmt);

  ::wxEndBusyCursor();

  ColumnStatsDialog dlg;
  dlg.Create(MainFrame, obj->GetName(), obj->GetColumn(), null_count,
             text_count, integer_count, real_count, blob_count, min, max, avg,
             stddev_pop, stddev_samp, var_pop, var_samp, distinct_values);
  dlg.ShowModal();
}

void MyTableTree::OnCmdDropColumn(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - drop column
//
  char column[1024];
  int ret;
  char **results;
  int rows;
  int columns;
  int i;
  char *errMsg = NULL;
  wxString sql;
  wxString createSql;
  wxString insertSql;
  wxString insertFromSql;
  wxString renameSql;
  wxString dropSql;
  wxString dropIndexSql;
  wxString createIndexSql;
  wxString disableSpatialIdxSql;
  wxString dropSpatialIdxSql;
  wxString createSpatialIdxSql;
  wxString discardGeometrySql;
  wxString addGeometrySql;
  wxString geomColumn;
  wxString geomType;
  wxString coordDims;
  int geomSrid;
  int indexType;
  wxString indexName;
  bool uniqueIndex;
  bool comma = false;
  char *value;
  bool check_autoincrement = false;
  bool autoincrement = false;
  wxString aliasTable;
  bool isGeom;
  wxString msg;
  char dummy[64];
  GeomColsList Geometries;
  GeomColumn *pG;
  TblIndexList Index;
  TblIndex *pI;
  IndexColumn *pC;
  wxString name;
  char xname[1024];
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;

// checking if the SQLITE_SEQUENCE table exists
  sql = wxT("PRAGMA table_info(sqlite_sequence)");
  ret = sqlite3_get_table(MainFrame->GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      return;
    }
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          check_autoincrement = true;
        }
    }
  sqlite3_free_table(results);
  if (check_autoincrement == true)
    {
// checking if there is an AUTOINCREMENT Primary Key
      sql = wxT("SELECT name FROM sqlite_sequence WHERE name LIKE '");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("'");
      ret = sqlite3_get_table(MainFrame->GetSqlite(), sql.ToUTF8(), &results,
                              &rows, &columns, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          return;
        }
      if (rows < 1)
        ;
      else
        {
          for (i = 1; i <= rows; i++)
            {
              autoincrement = true;
            }
        }
      sqlite3_free_table(results);
    }
// checking if there is some Spatial Index already defined
  sql =
    wxT
    ("SELECT f_geometry_column, type, coord_dimension, srid, spatial_index_enabled ");
  sql += wxT("FROM geometry_columns WHERE f_table_name LIKE '");
  strcpy(xname, obj->GetName().ToUTF8());
  MainFrame->CleanSqlString(xname);
  sql += wxString::FromUTF8(xname);
  sql += wxT("'");
  ret = sqlite3_get_table(MainFrame->GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      return;
    }
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          value = results[(i * columns) + 0];
          geomColumn = wxString::FromUTF8(value);
          value = results[(i * columns) + 1];
          geomType = wxString::FromUTF8(value);
          value = results[(i * columns) + 2];
          coordDims = wxString::FromUTF8(value);
          value = results[(i * columns) + 3];
          geomSrid = atoi(value);
          value = results[(i * columns) + 4];
          indexType = atoi(value);
          Geometries.Add(geomColumn, geomType, coordDims, geomSrid, indexType);
        }
    }
  sqlite3_free_table(results);

// creating the SQL fragments
  aliasTable = wxT("tmp_alias ");
  aliasTable += obj->GetName();
  aliasTable += wxT(" tmp_alias");
  strcpy(xname, aliasTable.ToUTF8());
  MainFrame->DoubleQuotedSql(xname);
  aliasTable = wxString::FromUTF8(xname);
  renameSql = wxT("ALTER TABLE ");
  strcpy(xname, obj->GetName().ToUTF8());
  MainFrame->DoubleQuotedSql(xname);
  renameSql += wxString::FromUTF8(xname);
  renameSql += wxT(" RENAME TO ");
  renameSql += aliasTable;
  renameSql += wxT(";\n");
  dropSql = wxT("DROP TABLE ");
  dropSql += aliasTable;
  dropSql += wxT(";\n");
  strcpy(column, obj->GetColumn().ToUTF8());
  pG = Geometries.GetFirst();
  while (pG)
    {
      if (pG->IsRTree() == true || pG->IsMbrCache() == true)
        {
          // disabling a Spatial Index
          disableSpatialIdxSql += wxT("SELECT DisableSpatialIndex('");
          strcpy(xname, obj->GetName().ToUTF8());
          MainFrame->CleanSqlString(xname);
          disableSpatialIdxSql += wxString::FromUTF8(xname);
          disableSpatialIdxSql += wxT("', '");
          strcpy(xname, pG->GetGeometryName().ToUTF8());
          MainFrame->CleanSqlString(xname);
          disableSpatialIdxSql += wxString::FromUTF8(xname);
          disableSpatialIdxSql += wxT("');\n");
          dropSpatialIdxSql += wxT("DROP TABLE IF EXISTS ");
          name = wxT("idx_");
          name += obj->GetName();
          name += wxT("_");
          name += pG->GetGeometryName();
          strcpy(xname, name.ToUTF8());
          MainFrame->DoubleQuotedSql(xname);
          dropSpatialIdxSql += wxString::FromUTF8(xname);
          dropSpatialIdxSql += wxT(";\n");
          if (pG->IsRTree() == true)
            {
              // creating an RTree Spatial Index
              createSpatialIdxSql += wxT("SELECT CreateSpatialIndex('");
              strcpy(xname, obj->GetName().ToUTF8());
              MainFrame->CleanSqlString(xname);
              createSpatialIdxSql += wxString::FromUTF8(xname);
              createSpatialIdxSql += wxT("', '");
              strcpy(xname, pG->GetGeometryName().ToUTF8());
              MainFrame->CleanSqlString(xname);
              createSpatialIdxSql += wxString::FromUTF8(xname);
              createSpatialIdxSql += wxT("');\n");
          } else
            {
              // creating an MbrCache Spatial Index
              createSpatialIdxSql += wxT("SELECT CreateMbrCache('");
              strcpy(xname, obj->GetName().ToUTF8());
              MainFrame->CleanSqlString(xname);
              createSpatialIdxSql += wxString::FromUTF8(xname);
              createSpatialIdxSql += wxT("', '");
              strcpy(xname, pG->GetGeometryName().ToUTF8());
              MainFrame->CleanSqlString(xname);
              createSpatialIdxSql += wxString::FromUTF8(xname);
              createSpatialIdxSql += wxT("');\n");
            }
        }
      // discarding a Geometry Column
      discardGeometrySql += wxT("SELECT DiscardGeometryColumn('");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      discardGeometrySql += wxString::FromUTF8(xname);
      discardGeometrySql += wxT("', '");
      strcpy(xname, pG->GetGeometryName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      discardGeometrySql += wxString::FromUTF8(xname);
      discardGeometrySql += wxT("');\n");
      pG = pG->GetNext();
    }

// retrieving the Column names
  sql = wxT("PRAGMA table_info(");
  strcpy(xname, obj->GetName().ToUTF8());
  MainFrame->DoubleQuotedSql(xname);
  sql += wxString::FromUTF8(xname);
  sql += wxT(")");
  ret = sqlite3_get_table(MainFrame->GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      return;
    }
  if (rows < 1)
    ;
  else
    {
      createSql = wxT("CREATE TABLE ");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      createSql += wxString::FromUTF8(xname);
      createSql += wxT(" (\n");
      insertSql = wxT("INSERT INTO ");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      insertSql += wxString::FromUTF8(xname);
      insertSql += wxT(" (");
      insertFromSql = wxT("SELECT ");
      for (i = 1; i <= rows; i++)
        {
          value = results[(i * columns) + 1];
          if (strcasecmp(value, column) == 0)
            continue;
          isGeom = false;
          pG = Geometries.GetFirst();
          while (pG)
            {
              char geom[1024];
              strcpy(geom, pG->GetGeometryName().ToUTF8());
              if (strcasecmp(geom, value) == 0)
                {
                  isGeom = true;
                  geomColumn = pG->GetGeometryName();
                  break;
                }
              pG = pG->GetNext();
            }
          if (comma == true)
            {
              if (isGeom == false)
                createSql += wxT(",\n");
              insertSql += wxT(", ");
              insertFromSql += wxT(", ");
            }
          if (isGeom == false)
            {
              strcpy(xname, value);
              MainFrame->DoubleQuotedSql(xname);
              createSql += wxString::FromUTF8(xname);
              createSql += wxT(" ");
            }
          strcpy(xname, value);
          MainFrame->DoubleQuotedSql(xname);
          insertSql += wxString::FromUTF8(xname);
          strcpy(xname, value);
          MainFrame->DoubleQuotedSql(xname);
          insertFromSql += wxString::FromUTF8(xname);
          value = results[(i * columns) + 2];
          if (isGeom == false)
            createSql += wxString::FromUTF8(value);
          value = results[(i * columns) + 5];
          if (value)
            {
              if (atoi(value) != 0)
                {
                  if (isGeom == false)
                    {
                      createSql += wxT(" PRIMARY KEY");
                      if (autoincrement == true)
                        createSql += wxT(" AUTOINCREMENT");
                    }
                }
            }
          value = results[(i * columns) + 3];
          if (value)
            {
              if (atoi(value) != 0)
                {
                  if (isGeom == true)
                    Geometries.SetNotNull(geomColumn);
                  else
                    createSql += wxT(" NOT NULL");
                }
            }
          value = results[(i * columns) + 4];
          if (value && isGeom == false)
            {
              createSql += wxT(" DEFAULT ");
              createSql += wxString::FromUTF8(value);
            }
          comma = true;
        }
      createSql += wxT(");\n");
      insertSql += wxT(")\n");
      insertFromSql += wxT("\nFROM ");
      insertFromSql += aliasTable;
      insertFromSql += wxT(";\n");
      insertSql += insertFromSql;
    }
  sqlite3_free_table(results);

  pG = Geometries.GetFirst();
  while (pG)
    {
      // adding a Geometry Column
      addGeometrySql += wxT("SELECT AddGeometryColumn('");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      addGeometrySql += wxString::FromUTF8(xname);
      addGeometrySql += wxT("', '");
      strcpy(xname, pG->GetGeometryName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      addGeometrySql += wxString::FromUTF8(xname);
      sprintf(dummy, "', %d", pG->GetSrid());
      addGeometrySql += wxString::FromUTF8(dummy);
      addGeometrySql += wxT(", '");
      addGeometrySql += pG->GetGeometryType();
      if (pG->GetCoordDims() == wxT('2') || pG->GetCoordDims() == wxT('3'))
        {
          addGeometrySql += wxT("', ");
          addGeometrySql += pG->GetCoordDims();
      } else
        {
          addGeometrySql += wxT("', '");
          addGeometrySql += pG->GetCoordDims();
          addGeometrySql += wxT("'");
        }
      if (pG->IsNotNull() == false)
        addGeometrySql += wxT(");\n");
      else
        addGeometrySql += wxT(", 1);\n");
      pG = pG->GetNext();
    }

// retrieving any related Index
  sql = wxT("PRAGMA index_list(");
  strcpy(xname, obj->GetName().ToUTF8());
  MainFrame->DoubleQuotedSql(xname);
  sql += wxString::FromUTF8(xname);
  sql += wxT(")");
  ret = sqlite3_get_table(MainFrame->GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      return;
    }
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          value = results[(i * columns) + 1];
          indexName = wxString::FromUTF8(value);
          value = results[(i * columns) + 2];
          if (atoi(value) == 0)
            uniqueIndex = false;
          else
            uniqueIndex = true;
          Index.Add(indexName, uniqueIndex);
        }
    }
  sqlite3_free_table(results);
  pI = Index.GetFirst();
  while (pI)
    {
      // retrieving any Index Column
      sql = wxT("PRAGMA index_info(");
      strcpy(xname, pI->GetIndexName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT(")");
      ret = sqlite3_get_table(MainFrame->GetSqlite(), sql.ToUTF8(), &results,
                              &rows, &columns, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          return;
        }
      if (rows < 1)
        ;
      else
        {
          for (i = 1; i <= rows; i++)
            {
              value = results[(i * columns) + 2];
              indexName = wxString::FromUTF8(value);
              pI->Add(indexName);
            }
        }
      sqlite3_free_table(results);
      pI = pI->GetNext();
    }

// setting up the Index SQL fragments
  Index.Invalidate(obj->GetColumn());
  pI = Index.GetFirst();
  while (pI)
    {
      dropIndexSql += wxT("DROP INDEX ");
      strcpy(xname, pI->GetIndexName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      dropIndexSql += wxString::FromUTF8(xname);
      dropIndexSql += wxT(";\n");
      if (pI->IsValid() == true)
        {
          if (pI->GetFirst())
            {
              if (pI->IsUnique() == true)
                createIndexSql += wxT("CREATE UNIQUE INDEX ");
              else
                createIndexSql += wxT("CREATE INDEX ");
              strcpy(xname, pI->GetIndexName().ToUTF8());
              MainFrame->DoubleQuotedSql(xname);
              createIndexSql += wxString::FromUTF8(xname);
              createIndexSql += wxT(" ON ");
              strcpy(xname, obj->GetName().ToUTF8());
              MainFrame->DoubleQuotedSql(xname);
              createIndexSql += wxString::FromUTF8(xname);
              createIndexSql += wxT(" (");
            }
          comma = false;
          pC = pI->GetFirst();
          while (pC)
            {
              if (comma == true)
                createIndexSql += wxT(", ");
              strcpy(xname, pC->GetColumnName().ToUTF8());
              MainFrame->DoubleQuotedSql(xname);
              createIndexSql += wxString::FromUTF8(xname);
              comma = true;
              pC = pC->GetNext();
            }
          if (pI->GetFirst())
            createIndexSql += wxT(");\n");
        }
      pI = pI->GetNext();
    }

// setting up the SQL complex statement
  sql = wxT("BEGIN;\n");
  sql += disableSpatialIdxSql;
  sql += discardGeometrySql;
  sql += dropSpatialIdxSql;
  sql += dropIndexSql;
  sql += renameSql;
  sql += createSql;
  sql += addGeometrySql;
  sql += createSpatialIdxSql;
  sql += createIndexSql;
  sql += insertSql;
  sql += dropSql;
  sql += wxT("COMMIT;");
  if (sql.Len() < 1)
    return;
  msg = wxT("Do you really intend to drop the Column ");
  msg += obj->GetColumn();
  msg += wxT("\nfrom the Table ");
  msg += obj->GetName();
  msg += wxT(" ?");
  wxMessageDialog confirm(this, msg, wxT("Confirming DROP COLUMN"),
                          wxOK | wxCANCEL | wxICON_QUESTION);
  ret = confirm.ShowModal();
  if (ret != wxID_OK)
    return;

// executing
  ::wxBeginBusyCursor();
  ret = sqlite3_exec(MainFrame->GetSqlite(), sql.ToUTF8(), NULL, NULL, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      ::wxEndBusyCursor();
      goto rollback;
    }
  ::wxEndBusyCursor();
  wxMessageBox(wxT("The column ") + obj->GetColumn() +
               wxT("\nwas successfully removed\nfrom the Table ") +
               obj->GetName(), wxT("spatialite-gui"),
               wxOK | wxICON_INFORMATION, this);
  MainFrame->InitTableTree();
  return;
rollback:
  ret = sqlite3_exec(MainFrame->GetSqlite(), "ROLLBACK", NULL, NULL, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      ::wxEndBusyCursor();
      return;
    }
  ::wxEndBusyCursor();
  wxMessageBox(wxT
               ("An error occurred\n\na ROLLBACK was automatically performed"),
               wxT("spatialite-gui"), wxOK | wxICON_WARNING, this);
}

void MyTableTree::OnCmdRenameColumn(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - rename column
//
  char column[1024];
  int ret;
  char **results;
  int rows;
  int columns;
  int i;
  char *errMsg = NULL;
  wxString sql;
  wxString createSql;
  wxString insertSql;
  wxString insertFromSql;
  wxString renameSql;
  wxString dropSql;
  wxString dropIndexSql;
  wxString createIndexSql;
  wxString disableSpatialIdxSql;
  wxString dropSpatialIdxSql;
  wxString createSpatialIdxSql;
  wxString discardGeometrySql;
  wxString addGeometrySql;
  wxString geomColumn;
  wxString geomType;
  wxString coordDims;
  int geomSrid;
  int indexType;
  wxString indexName;
  bool uniqueIndex;
  bool comma = false;
  char *value;
  bool check_autoincrement = false;
  bool autoincrement = false;
  wxString aliasTable;
  bool isGeom;
  wxString msg;
  char dummy[64];
  GeomColsList Geometries;
  GeomColumn *pG;
  TblIndexList Index;
  TblIndex *pI;
  IndexColumn *pC;
  wxString name;
  char xname[1024];
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;

// asking the new column name
  wxString newColumn =
    wxGetTextFromUser(wxT("Please, insert the new Column Name"),
                      wxT("Rename Column"), obj->GetColumn(),
                      MainFrame, wxDefaultCoord, wxDefaultCoord, false);

// checking if the SQLITE_SEQUENCE table exists
  sql = wxT("PRAGMA table_info(sqlite_sequence)");
  ret = sqlite3_get_table(MainFrame->GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      return;
    }
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          check_autoincrement = true;
        }
    }
  sqlite3_free_table(results);
  if (check_autoincrement == true)
    {
// checking if there is an AUTOINCREMENT Primary Key
      sql = wxT("SELECT name FROM sqlite_sequence WHERE name LIKE '");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT("'");
      ret = sqlite3_get_table(MainFrame->GetSqlite(), sql.ToUTF8(), &results,
                              &rows, &columns, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          return;
        }
      if (rows < 1)
        ;
      else
        {
          for (i = 1; i <= rows; i++)
            {
              autoincrement = true;
            }
        }
      sqlite3_free_table(results);
    }
// checking if there is some Spatial Index already defined
  sql =
    wxT
    ("SELECT f_geometry_column, type, coord_dimension, srid, spatial_index_enabled ");
  sql += wxT("FROM geometry_columns WHERE f_table_name LIKE '");
  strcpy(xname, obj->GetName().ToUTF8());
  MainFrame->CleanSqlString(xname);
  sql += wxString::FromUTF8(xname);
  sql += wxT("'");
  ret = sqlite3_get_table(MainFrame->GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      return;
    }
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          value = results[(i * columns) + 0];
          geomColumn = wxString::FromUTF8(value);
          value = results[(i * columns) + 1];
          geomType = wxString::FromUTF8(value);
          value = results[(i * columns) + 2];
          coordDims = wxString::FromUTF8(value);
          value = results[(i * columns) + 3];
          geomSrid = atoi(value);
          value = results[(i * columns) + 4];
          indexType = atoi(value);
          Geometries.Add(geomColumn, geomType, coordDims, geomSrid, indexType);
        }
    }
  sqlite3_free_table(results);

// creating the SQL fragments
  aliasTable = wxT("tmp_alias ");
  aliasTable += obj->GetName();
  aliasTable += wxT(" tmp_alias");
  strcpy(xname, aliasTable.ToUTF8());
  MainFrame->DoubleQuotedSql(xname);
  aliasTable = wxString::FromUTF8(xname);
  renameSql = wxT("ALTER TABLE ");
  strcpy(xname, obj->GetName().ToUTF8());
  MainFrame->DoubleQuotedSql(xname);
  renameSql += wxString::FromUTF8(xname);
  renameSql += wxT(" RENAME TO ");
  renameSql += aliasTable;
  renameSql += wxT(";\n");
  dropSql = wxT("DROP TABLE ");
  dropSql += aliasTable;
  dropSql += wxT(";\n");
  strcpy(column, obj->GetColumn().ToUTF8());
  pG = Geometries.GetFirst();
  while (pG)
    {
      if (pG->IsRTree() == true || pG->IsMbrCache() == true)
        {
          // disabling a Spatial Index
          disableSpatialIdxSql += wxT("SELECT DisableSpatialIndex('");
          strcpy(xname, obj->GetName().ToUTF8());
          MainFrame->CleanSqlString(xname);
          disableSpatialIdxSql += wxString::FromUTF8(xname);
          disableSpatialIdxSql += wxT("', '");
          strcpy(xname, pG->GetGeometryName().ToUTF8());
          MainFrame->CleanSqlString(xname);
          disableSpatialIdxSql += wxString::FromUTF8(xname);
          disableSpatialIdxSql += wxT("');\n");
          dropSpatialIdxSql += wxT("DROP TABLE IF EXISTS ");
          name = wxT("idx_");
          name += obj->GetName();
          name += wxT("_");
          name += pG->GetGeometryName();
          strcpy(xname, name.ToUTF8());
          MainFrame->DoubleQuotedSql(xname);
          dropSpatialIdxSql += wxString::FromUTF8(xname);
          dropSpatialIdxSql += wxT(";\n");
          if (pG->IsRTree() == true)
            {
              // creating an RTree Spatial Index
              createSpatialIdxSql += wxT("SELECT CreateSpatialIndex('");
              strcpy(xname, obj->GetName().ToUTF8());
              MainFrame->CleanSqlString(xname);
              createSpatialIdxSql += wxString::FromUTF8(xname);
              createSpatialIdxSql += wxT("', '");
              strcpy(xname, pG->GetGeometryName().ToUTF8());
              MainFrame->CleanSqlString(xname);
              createSpatialIdxSql += wxString::FromUTF8(xname);
              createSpatialIdxSql += wxT("');\n");
          } else
            {
              // creating an MbrCache Spatial Index
              createSpatialIdxSql += wxT("SELECT CreateMbrCache('");
              strcpy(xname, obj->GetName().ToUTF8());
              MainFrame->CleanSqlString(xname);
              createSpatialIdxSql += wxString::FromUTF8(xname);
              createSpatialIdxSql += wxT("', '");
              strcpy(xname, pG->GetGeometryName().ToUTF8());
              MainFrame->CleanSqlString(xname);
              createSpatialIdxSql += wxString::FromUTF8(xname);
              createSpatialIdxSql += wxT("');\n");
            }
        }
      // discarding a Geometry Column
      discardGeometrySql += wxT("SELECT DiscardGeometryColumn('");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      discardGeometrySql += wxString::FromUTF8(xname);
      discardGeometrySql += wxT("', '");
      strcpy(xname, pG->GetGeometryName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      discardGeometrySql += wxString::FromUTF8(xname);
      discardGeometrySql += wxT("');\n");
      pG = pG->GetNext();
    }

// retrieving the Column names
  sql = wxT("PRAGMA table_info(");
  strcpy(xname, obj->GetName().ToUTF8());
  MainFrame->DoubleQuotedSql(xname);
  sql += wxString::FromUTF8(xname);
  sql += wxT(")");
  ret = sqlite3_get_table(MainFrame->GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      return;
    }
  if (rows < 1)
    ;
  else
    {
      createSql = wxT("CREATE TABLE ");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      createSql += wxString::FromUTF8(xname);
      createSql += wxT(" (\n");
      insertSql = wxT("INSERT INTO ");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      insertSql += wxString::FromUTF8(xname);
      insertSql += wxT(" (");
      insertFromSql = wxT("SELECT ");
      for (i = 1; i <= rows; i++)
        {
          value = results[(i * columns) + 1];
          isGeom = false;
          pG = Geometries.GetFirst();
          while (pG)
            {
              char geom[1024];
              strcpy(geom, pG->GetGeometryName().ToUTF8());
              if (strcasecmp(geom, value) == 0)
                {
                  isGeom = true;
                  geomColumn = pG->GetGeometryName();
                  break;
                }
              pG = pG->GetNext();
            }
          if (comma == true)
            {
              if (isGeom == false)
                createSql += wxT(",\n");
              insertSql += wxT(", ");
              insertFromSql += wxT(", ");
            }
          if (isGeom == false)
            {
              if (strcasecmp(value, column) == 0)
                {
                  strcpy(xname, newColumn.ToUTF8());
                  MainFrame->DoubleQuotedSql(xname);
                  createSql += wxString::FromUTF8(xname);
              } else
                {
                  strcpy(xname, value);
                  MainFrame->DoubleQuotedSql(xname);
                  createSql += wxString::FromUTF8(xname);
                }
              createSql += wxT(" ");
            }
          if (strcasecmp(value, column) == 0)
            {
              strcpy(xname, newColumn.ToUTF8());
              MainFrame->DoubleQuotedSql(xname);
              insertSql += wxString::FromUTF8(xname);
          } else
            {
              strcpy(xname, value);
              MainFrame->DoubleQuotedSql(xname);
              insertSql += wxString::FromUTF8(xname);
            }
          strcpy(xname, value);
          MainFrame->DoubleQuotedSql(xname);
          insertFromSql += wxString::FromUTF8(xname);
          value = results[(i * columns) + 2];
          if (isGeom == false)
            createSql += wxString::FromUTF8(value);
          value = results[(i * columns) + 5];
          if (value)
            {
              if (atoi(value) != 0)
                {
                  if (isGeom == false)
                    {
                      createSql += wxT(" PRIMARY KEY");
                      if (autoincrement == true)
                        createSql += wxT(" AUTOINCREMENT");
                    }
                }
            }
          value = results[(i * columns) + 3];
          if (value)
            {
              if (atoi(value) != 0)
                {
                  if (isGeom == true)
                    Geometries.SetNotNull(geomColumn);
                  else
                    createSql += wxT(" NOT NULL");
                }
            }
          value = results[(i * columns) + 4];
          if (value && isGeom == false)
            {
              createSql += wxT(" DEFAULT ");
              createSql += wxString::FromUTF8(value);
            }
          comma = true;
        }
      createSql += wxT(");\n");
      insertSql += wxT(")\n");
      insertFromSql += wxT("\nFROM ");
      insertFromSql += aliasTable;
      insertFromSql += wxT(";\n");
      insertSql += insertFromSql;
    }
  sqlite3_free_table(results);

  pG = Geometries.GetFirst();
  while (pG)
    {
      // adding a Geometry Column
      addGeometrySql += wxT("SELECT AddGeometryColumn('");
      strcpy(xname, obj->GetName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      addGeometrySql += wxString::FromUTF8(xname);
      addGeometrySql += wxT("', '");
      strcpy(xname, pG->GetGeometryName().ToUTF8());
      MainFrame->CleanSqlString(xname);
      addGeometrySql += wxString::FromUTF8(xname);
      sprintf(dummy, "', %d", pG->GetSrid());
      addGeometrySql += wxString::FromUTF8(dummy);
      addGeometrySql += wxT(", '");
      addGeometrySql += pG->GetGeometryType();
      addGeometrySql += wxT("', '");
      addGeometrySql += pG->GetCoordDims();
      if (pG->IsNotNull() == false)
        addGeometrySql += wxT("');\n");
      else
        addGeometrySql += wxT("', 1);\n");
      pG = pG->GetNext();
    }

// retrieving any related Index
  sql = wxT("PRAGMA index_list(");
  strcpy(xname, obj->GetName().ToUTF8());
  MainFrame->DoubleQuotedSql(xname);
  sql += wxString::FromUTF8(xname);
  sql += wxT(")");
  ret = sqlite3_get_table(MainFrame->GetSqlite(), sql.ToUTF8(), &results,
                          &rows, &columns, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      return;
    }
  if (rows < 1)
    ;
  else
    {
      for (i = 1; i <= rows; i++)
        {
          value = results[(i * columns) + 1];
          indexName = wxString::FromUTF8(value);
          value = results[(i * columns) + 2];
          if (atoi(value) == 0)
            uniqueIndex = false;
          else
            uniqueIndex = true;
          Index.Add(indexName, uniqueIndex);
        }
    }
  sqlite3_free_table(results);
  pI = Index.GetFirst();
  while (pI)
    {
      // retrieving any Index Column
      sql = wxT("PRAGMA index_info(");
      strcpy(xname, pI->GetIndexName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      sql += wxString::FromUTF8(xname);
      sql += wxT(")");
      ret = sqlite3_get_table(MainFrame->GetSqlite(), sql.ToUTF8(), &results,
                              &rows, &columns, &errMsg);
      if (ret != SQLITE_OK)
        {
          wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                       wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
          sqlite3_free(errMsg);
          return;
        }
      if (rows < 1)
        ;
      else
        {
          for (i = 1; i <= rows; i++)
            {
              value = results[(i * columns) + 2];
              indexName = wxString::FromUTF8(value);
              pI->Add(indexName);
            }
        }
      sqlite3_free_table(results);
      pI = pI->GetNext();
    }

// setting up the Index SQL fragments
  pI = Index.GetFirst();
  while (pI)
    {
      dropIndexSql += wxT("DROP INDEX ");
      strcpy(xname, pI->GetIndexName().ToUTF8());
      MainFrame->DoubleQuotedSql(xname);
      dropIndexSql += wxString::FromUTF8(xname);
      dropIndexSql += wxT(";\n");
      if (pI->IsValid() == true)
        {
          if (pI->GetFirst())
            {
              if (pI->IsUnique() == true)
                createIndexSql += wxT("CREATE UNIQUE INDEX ");
              else
                createIndexSql += wxT("CREATE INDEX ");
              strcpy(xname, pI->GetIndexName().ToUTF8());
              MainFrame->DoubleQuotedSql(xname);
              createIndexSql += wxString::FromUTF8(xname);
              createIndexSql += wxT(" ON ");
              strcpy(xname, obj->GetName().ToUTF8());
              MainFrame->DoubleQuotedSql(xname);
              createIndexSql += wxString::FromUTF8(xname);
              createIndexSql += wxT(" (");
            }
          comma = false;
          pC = pI->GetFirst();
          while (pC)
            {
              char xvalue[1024];
              strcpy(xvalue, pC->GetColumnName().ToUTF8());
              if (comma == true)
                createIndexSql += wxT(", ");
              if (strcasecmp(xvalue, column) == 0)
                {
                  strcpy(xname, newColumn.ToUTF8());
                  MainFrame->DoubleQuotedSql(xname);
                  createIndexSql += wxString::FromUTF8(xname);
              } else
                {
                  strcpy(xname, pC->GetColumnName().ToUTF8());
                  MainFrame->DoubleQuotedSql(xname);
                  createIndexSql += wxString::FromUTF8(xname);
                }
              comma = true;
              pC = pC->GetNext();
            }
          if (pI->GetFirst())
            createIndexSql += wxT(");\n");
        }
      pI = pI->GetNext();
    }

// setting up the SQL complex statement
  sql = wxT("BEGIN;\n");
  sql += disableSpatialIdxSql;
  sql += discardGeometrySql;
  sql += dropSpatialIdxSql;
  sql += dropIndexSql;
  sql += renameSql;
  sql += createSql;
  sql += addGeometrySql;
  sql += createSpatialIdxSql;
  sql += createIndexSql;
  sql += insertSql;
  sql += dropSql;
  sql += wxT("COMMIT;");
  if (sql.Len() < 1)
    return;
  msg = wxT("Do you really intend to rename the Column ");
  msg += obj->GetColumn();
  msg += wxT(" as ");
  msg += newColumn;
  msg += wxT("\ninto the Table ");
  msg += obj->GetName();
  msg += wxT(" ?");
  wxMessageDialog confirm(this, msg, wxT("Confirming RENAME COLUMN"),
                          wxOK | wxCANCEL | wxICON_QUESTION);
  ret = confirm.ShowModal();
  if (ret != wxID_OK)
    return;

// executing
  ::wxBeginBusyCursor();
  ret = sqlite3_exec(MainFrame->GetSqlite(), sql.ToUTF8(), NULL, NULL, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      ::wxEndBusyCursor();
      goto rollback;
    }
  ::wxEndBusyCursor();
  wxMessageBox(wxT("The column ") + obj->GetColumn() +
               wxT("\nwas successfully renamed as ") + newColumn +
               wxT("\ninto the Table ") + obj->GetName(),
               wxT("spatialite-gui"), wxOK | wxICON_INFORMATION, this);
  MainFrame->InitTableTree();
  return;
rollback:
  ret = sqlite3_exec(MainFrame->GetSqlite(), "ROLLBACK", NULL, NULL, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      ::wxEndBusyCursor();
      return;
    }
  ::wxEndBusyCursor();
  wxMessageBox(wxT
               ("An error occurred\n\na ROLLBACK was automatically performed"),
               wxT("spatialite-gui"), wxOK | wxICON_WARNING, this);
}

void MyTableTree::OnCmdEdit(wxCommandEvent & WXUNUSED(event))
{
//
// menu event - editing row values
//
  char **results;
  int rows;
  int columns;
  int i;
  char *errMsg = NULL;
  wxString sql;
  char *column;
  char *type;
  int pk = 0;
  int pb = 0;
  int primaryKeys[1024];
  int blobCols[1024];
  char xname[1024];
  for (i = 0; i < 1024; i++)
    {
      primaryKeys[i] = -1;
      blobCols[i] = -1;
    }
  primaryKeys[pk++] = 0;
  MyObject *obj = (MyObject *) GetItemData(CurrentItem);
  if (obj == NULL)
    return;
  sql = wxT("PRAGMA table_info(");
  strcpy(xname, obj->GetName().ToUTF8());
  MainFrame->DoubleQuotedSql(xname);
  sql += wxString::FromUTF8(xname);
  sql += wxT(")");
  int ret = sqlite3_get_table(MainFrame->GetSqlite(), sql.ToUTF8(), &results,
                              &rows, &columns, &errMsg);
  if (ret != SQLITE_OK)
    {
      wxMessageBox(wxT("SQLite SQL error: ") + wxString::FromUTF8(errMsg),
                   wxT("spatialite-gui"), wxOK | wxICON_ERROR, this);
      sqlite3_free(errMsg);
      return;
    }
  if (rows < 1)
    ;
  else
    {
      sql = wxT("SELECT ROWID");
      for (i = 1; i <= rows; i++)
        {
          column = results[(i * columns) + 1];
          sql += wxT(", ");
          strcpy(xname, column);
          MainFrame->DoubleQuotedSql(xname);
          sql += wxString::FromUTF8(xname);
          type = results[(i * columns) + 2];
          if (strcasecmp(type, "BLOB") == 0)
            blobCols[pb++] = i;
          if (atoi(results[(i * columns) + 5]) == 0)
            ;
          else
            primaryKeys[pk++] = i;
        }
    }
  sqlite3_free_table(results);
  if (sql.Len() < 1)
    return;
  sql += wxT("\nFROM ");
  strcpy(xname, obj->GetName().ToUTF8());
  MainFrame->DoubleQuotedSql(xname);
  sql += wxString::FromUTF8(xname);
  sql += wxT("\nORDER BY ROWID");
  MainFrame->EditTable(sql, primaryKeys, blobCols, obj->GetName());
}
